# have to be first line
from qiskit import Aer

import numpy
from pyscf import gto, scf, ao2mo
from pygrisb.math.matrix_util import unitary_transform_coulomb_matrix

from qiskit_aqua import QuantumInstance
from qiskit_aqua.algorithms import VQE, ExactEigensolver
from qiskit_aqua.components.optimizers import COBYLA
from qiskit_chemistry import FermionicOperator

from qiskit_chemistry.aqua_extensions.components.variational_forms \
        import UCCSD
from qiskit_chemistry.aqua_extensions.components.initial_states \
        import HartreeFock

from qiskit import IBMQ
from qiskit.providers.ibmq import least_busy
IBMQ.load_accounts()

import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)


def computer_hf_integrals(h1e, h2e):
    mol = gto.M()
    n = h1e.shape[0]
    # half filling
    mol.nelectron = n
    # setup hartree-fock calculation
    mf = scf.RHF(mol)
    mf.get_hcore = lambda *args: h1e
    mf.get_ovlp = lambda *args: numpy.eye(n)
    # get 8-fold permutation symmetry of the integrals
    mf._eri = ao2mo.restore(8, h2e, n)
    mf.kernel()
    # one and two-body integrals with mo
    mohij = mf.mo_coeff.T.dot(h1e).dot(mf.mo_coeff)
    movijkl = unitary_transform_coulomb_matrix(h2e, mf.mo_coeff)
    # adding spin
    ns = n*2
    mohsij = numpy.zeros([ns,ns])
    mohsij[:n,:n] = mohsij[n:,n:] = mohij
    movsijkl = numpy.zeros([ns,ns,ns,ns])
    movsijkl[:n,:n,:n,:n] = movsijkl[:n,:n,n:,n:] = movsijkl[n:,n:,:n,:n] = \
            movsijkl[n:,n:,n:,n:] = 0.5*movijkl
    return mf.mo_coeff, mohsij, movsijkl


def get_qubit_operator(h1, h2=None, qubit_reduction=False, map_type="parity",
        threshold=1.e-10):
    # half-filling
    num_particles = h1.shape[0]/2
    fop = FermionicOperator(h1=h1, h2=h2)
    qop = fop.mapping(map_type=map_type, threshold=threshold)
    if qubit_reduction:
        qop = qop.two_qubit_reduced_operator(num_particles)
    qop.chop(threshold)
    return qop


def check_exact_diag(qop, aux_qops=None):
    exact_eigensolver = ExactEigensolver(qop, k=1, aux_operators=aux_qops)
    ret = exact_eigensolver.run()
    print(ret['aux_ops'])
    print('The computed energy is: {:.12f}'.format(ret['eigvals'][0].real))


def qc_solver(h_qop, num_spin_orbitals, num_particles, map_type, \
        qubit_reduction, aux_qops=None):
    # backends = Aer.backends()
    backends = IBMQ.backends(simulator=False)
    print(backends)
    # backend = IBMQ.get_backend('ibmq_qasm_simulator')
    # backend = IBMQ.get_backend('ibmq_16_melbourne')

    # backend = Aer.get_backend('statevector_simulator')
    backend = Aer.get_backend('qasm_simulator')

    # setup COBYLA optimizer
    max_eval = 1000
    cobyla = COBYLA(maxiter=max_eval)
    # setup hartreeFock state
    hf_state = HartreeFock(h_qop.num_qubits, num_spin_orbitals, num_particles,
            map_type, qubit_reduction)
    # setup UCCSD variational form
    var_form = UCCSD(h_qop.num_qubits, depth=1,
            num_orbitals=num_spin_orbitals, num_particles=num_particles,
            active_occupied=[0], active_unoccupied=[0],
            initial_state=hf_state, qubit_mapping=map_type,
            two_qubit_reduction=qubit_reduction, num_time_slices=1)

    # setup VQE
    vqe = VQE(h_qop, var_form, cobyla, operator_mode='matrix', \
            aux_operators=aux_qops)
    quantum_instance = QuantumInstance(backend=backend, shots=1024,
            max_credits=10)
    ret = vqe.run(quantum_instance)
    print(ret['aux_ops'])
    print('The computed ground state energy is: {:.12f}'.format(\
            ret['eigvals'][0]))


def get_dm_qops(mo_coeff, qubit_reduction, map_type):
    n = mo_coeff.shape[0]
    ns = n*2
    qops = []
    for i in range(n):
        for j in range(i+1):
            h1 = numpy.zeros([n, n])
            h1[i,j] = 1.0
            mo_h1 = mo_coeff.T.conj().dot(h1).dot(mo_coeff)
            mo_h1s = numpy.zeros([ns, ns])
            mo_h1s[:n,:n] = mo_h1s[n:,n:] = mo_h1
            qops.append(get_qubit_operator(mo_h1s, \
                    qubit_reduction=qubit_reduction, \
                    map_type=map_type))
    return qops


def qcalc_e_dm(h1e, h2e, check_ed=False, map_type="parity"):
    h1e = numpy.asarray(h1e)
    h2e = numpy.asarray(h2e)
    num_spin_orbitals = h1e.shape[0]*2
    # half-filling
    num_particles = h1e.shape[0]
    if  map_type == "parity":
        qubit_reduction = True
    else:
        qubit_reduction = False
    # get integrals in molecular spin-orbital basis.
    mo_coeff, mo_h1, mo_h2 = computer_hf_integrals(h1e, h2e)
    h_qop = get_qubit_operator(mo_h1, mo_h2, qubit_reduction=qubit_reduction,\
            map_type=map_type)
    # density matrix operator
    aux_qops = get_dm_qops(mo_coeff, qubit_reduction, map_type)
    if check_ed:
        check_exact_diag(h_qop, aux_qops=aux_qops)
    qc_solver(h_qop, num_spin_orbitals, num_particles, map_type,
            qubit_reduction, aux_qops=aux_qops)



if __name__=="__main__":
    # n = 2
    # h1e = numpy.zeros([n ,n])
    # h1e[0,1] = h1e[1,0] = -1.
    # h2e = numpy.zeros([n, n, n, n])
    # h2e[0,0,0,0] = 2.
    # qcalc_e_dm(h1e, h2e, check_ed=True)
    n = 4
    h1e = numpy.zeros([n ,n])
    h1e[0,2] = h1e[2,0] = h1e[1,3] = h1e[3,1] = -1.
    h2e = numpy.zeros([n, n, n, n])
    h2e[0,0,0,0] = h2e[1,1,1,1] = 2.
    qcalc_e_dm(h1e, h2e, check_ed=True)



from . import basic
from . import estructure
from . import gutz
from . import gsolver
from . import io
from . import symm
from . import math
from . import mbody
from . import model
from . import dft
from . import iface
from . import run
from . import compatible



'''
Get lcoal operators given the {n_ij} operators.
'''

import h5py, pkg_resources
import itertools as it
import numpy as np
from scipy.sparse import csc_matrix
from pygrisb.io.h5io import get_coo_matrix


def get_linear_operators(nij, uij):
    '''
    Calculate \sum_{i,j} nij_{i,j}*uij(i,j).
    '''
    if nij is None:
        return None
    res = None
    for i, n1, u1 in it.izip(it.count(), nij, uij):
        for j, n12, u12 in it.izip(it.count(), n1, u1):
            if n12 is None:
                continue
            if abs(u12)>1.e-12:
                if res is None:
                    res = u12*n12
                else:
                    res += u12*n12
            if i==j:
                continue
            if abs(uij[j,i])>1.e-12:
                if res is None:
                    res = uij[j,i]*n12.getH()
                else:
                    res += uij[j,i]*n12.getH()
    if res is None:
        res = csc_matrix((1,1))
    return res


def get_am_op_from_nij(svec=None, lvec=None, jvec=None,
        nij=None, op_list=None):
    '''
    Get angular momentum operators given the {n_ij} operators and
    coiefficient matrice of spin and orbital momentum operators.
    '''
    if op_list is None:
        return None

    if svec is not None:
        s_z = get_linear_operators(nij, svec[2])
        s_p = get_linear_operators(nij, svec[0]+1.j*svec[1])
    if lvec is not None:
        l_z = get_linear_operators(nij, lvec[2])
        l_p = get_linear_operators(nij, lvec[0]+1.j*lvec[1])
    if svec is not None and lvec is not None:
        j_z, j_p = s_z + l_z, s_p + l_p
    elif jvec is not None:
        j_z = get_linear_operators(nij, jvec[2])
        j_p = get_linear_operators(nij, jvec[0]+1.j*jvec[1])

    res = {}

    if "Sx" in op_list:
        res["Sx"] = (s_p.getH() + s_p) / 2.0
    if "Sy" in op_list:
        res["Sy"] = (s_p - s_p.getH()) / 2.0j
    if "Sz" in op_list:
        res["Sz"] = s_z
    if "Lx" in op_list:
        res["Lx"] = (l_p.getH() + l_p) / 2.0
    if "Ly" in op_list:
        res["Ly"] = (l_p - l_p.getH()) / 2.0j
    if "Lz" in op_list:
        res["Lz"] = l_z
    if "Jx" in op_list:
        res["Jx"] = (j_p.getH() + j_p) / 2.0
    if "Jy" in op_list:
        res["Jy"] = (j_p - j_p.getH()) / 2.0j
    if "Jz" in op_list:
        res["Jz"] = j_z
    if "S2" in op_list:
        res["S2"] = s_p.getH() * s_p + s_z * s_z + s_z
    if "L2" in op_list:
        res["L2"] = l_p.getH() * l_p + l_z * l_z + l_z
    if "J2" in op_list:
        res["J2"] = j_p.getH() * j_p + j_z * j_z + j_z
    for key in res.keys():
        res[key] = res[key].tocsc()
    return res


def get_local_operators(imp, ival, op_list):
    '''
    Get the local operators, like Sx, Sy, Sz, Lx, Ly, Lz, S^2, L^2
    and J^2 operators for impurity imp.
    '''
    # Read in coefficient matrices for S and L.
    with h5py.File('GPARAM.h5', 'r') as f:
        svec = []
        svec.append(f['/IMPURITY_'+str(imp)+'/SX'][...].T)
        svec.append(f['/IMPURITY_'+str(imp)+'/SY'][...].T)
        svec.append(f['/IMPURITY_'+str(imp)+'/SZ'][...].T)
        lvec = []
        lvec.append(f['/IMPURITY_'+str(imp)+'/LX'][...].T)
        lvec.append(f['/IMPURITY_'+str(imp)+'/LY'][...].T)
        lvec.append(f['/IMPURITY_'+str(imp)+'/LZ'][...].T)

    # Read n_ij operators
    norb = svec[0].shape[0]
    nij = []
    with h5py.File('EMBED_HAMIL_ANALYSIS_'+str(imp)+'.h5', 'r') as f:
        for i in range(norb):
            n_i = []
            for j in range(i+1):
                path = '/valence_block_{}/NP_{}_{}'.format(ival,i+1,j+1)
                if path in f:
                    n_i.append(get_coo_matrix(f, path).tocsc())
                else:
                    n_i.append(None)
            nij.append(n_i)

    return get_am_op_from_nij(svec=svec, lvec=lvec, nij=nij, op_list=op_list)


def get_label_list(J, vecs):
    '''
    Get list of expection values of J for vecs.
    '''
    if J is None:
        label = None
    else:
        label = []
        for vec in vecs:
            res = np.vdot(vec, J.dot(vec))
            assert np.abs(res.imag) < 1.e-8, " error in get_label_list!"
            label.append(res.real)
        label = np.array(label)
    return label


def get_nij_op(l='d', ival='1'):
    '''get the matrix representation of n_{i,j}=c_{i}^{\dagger} c_{j}
    in the Hilbert space with valence of ival from preset file 'mbody.h5'.
    '''
    if l == 'd':
        norb = 10
    elif l == 'f':
        norb = 14
    else:
        raise ValueError('unsupported orbital={}!'.format(l))
    fname = pkg_resources.resource_filename('pygrisb', './mbody/mbody.h5')
    with h5py.File(fname, 'r') as f:
        nij = []
        for i in range(norb):
            n_i = []
            for j in range(i+1):
                path = '/{}/valence_block_{}/n_{}_{}'.format(
                        l,ival,i,j)
                n_i.append(get_coo_matrix(f, path).tocsc())
            nij.append(n_i)
    return nij


def get_lvec_op(lvec, l='d', ival='1'):
    '''get the l angular momentum operators [lx, ly, lz]
    in the Hilbert space with valence of ival,
    given their representation in single particle space (lvec).
    '''
    nij = get_nij_op(l=l, ival=ival)
    lops = get_am_op_from_nij(lvec=lvec, nij=nij,
            op_list=['Lx', 'Ly', 'Lz'])
    return [lops['Lx'], lops['Ly'], lops['Lz']]


def get_lrot_op(lvec, lie_jeven_params, l='d', ival='1'):
    '''get the 3d rotation matrix representations
    in the Hilbert space with valence of ival,
    given the single particle space representation of the l-angular momentum
    operators and lie parameters.
    '''
    lops = get_lvec_op(lvec, l=l, ival=ival)
    from pygrisb.symm.atom_symm import get_representation
    reps = get_representation(lops, lie_jeven_params)
    return reps



import h5py
import numpy as np


np.set_printoptions(precision=4,suppress=True, linewidth=100)

with h5py.File('glog.h5', 'r') as f:
    rmat = f['/Impurity_1/GA_R'][:3,:3]
    lmat = f['/Impurity_1/GA_La'][:3,:3]

umat = np.array([
    [0, 1, 0],
    [1, 0, 0],
    [0, 0, 1]], dtype=complex)

rmat = umat.T.conj().dot(rmat.dot(umat))
lmat = umat.T.conj().dot(lmat.dot(umat))

rmat2 = np.zeros((6, 6), dtype=complex)
rmat2[0::2, 0::2] = rmat
rmat2[1::2, 1::2] = rmat2[0::2, 0::2]

lmat2 = np.zeros((6, 6), dtype=complex)
lmat2[0::2, 0::2] = lmat
lmat2[1::2, 1::2] = lmat2[0::2, 0::2]

with h5py.File('WH_RL_INIT.h5', 'w') as f:
    f['/IMPURITY_1/LAMBDA'] = lmat2
    f['/IMPURITY_1/R'] = rmat2



import sys, numpy
from pymatgen import Molecule


'''
Help funtions for extracting molecule from crystal and determine local rotational operations.
'''

# module global molecule index
imol = 0

class gmolecule(Molecule):
    '''adding equivalent indices to atoms.
    '''
    def __init__(self, species, coords, equivalent_indices=[], charge=0,
            fm_direction=None, spin_multiplicity=None,
            validate_proximity=False, site_properties=None):
        super(Molecule, self).__init__(species, coords, charge=charge,
                spin_multiplicity=spin_multiplicity,
                validate_proximity=validate_proximity,
                site_properties=site_properties)
        self.fm_direction = fm_direction
        self.equivalent_indices = equivalent_indices


def xtal_get_local_rot(symbols, scaled_positions, cell, iat, dist_cut_,
        equivalent_indices=[], locrot=None, fm_direction=None,
        Nmax=4, tol=1.e-5, log=sys.stdout):
    '''
    Get rotation operations of center atom iat in crystal.
    '''
    mol = xtal_extract_mol(symbols, scaled_positions, cell, iat,
            dist_cut_, equivalent_indices=equivalent_indices,
            locrot=locrot, fm_direction=fm_direction, Nmax=Nmax)
    return mol_get_rot_list(mol, tol=tol, log=log)


def xtal_extract_mol(symbols, scaled_positions, cell, iat, dist_cut_,
        locrot=None, fm_direction=None, Nmax=10, equivalent_indices=[]):
    '''
    Extracting molecule of center atom iat from crystal.
    atom iat is the molecule center
    '''
    center = numpy.copy(scaled_positions[iat])
    new_scaled_positions = numpy.copy(scaled_positions)
    new_scaled_positions = new_scaled_positions - center
    molecule_positions = []
    molecule_symbols = []
    pair_dist = []
    eq_indices = []
    # Get the (2*Nmax+1)x(2*Nmax+1)x(2*Nmax+1) block
    Nmax_list = numpy.arange(-Nmax, Nmax)
    for i in Nmax_list:
        for j in Nmax_list:
            for k in Nmax_list:
                for jat, sp in enumerate(new_scaled_positions):
                    n_sp = sp + numpy.array([i, j, k])
                    n_p = \
                            n_sp[0] * numpy.array(cell[0]) + \
                            n_sp[1] * numpy.array(cell[1]) + \
                            n_sp[2] * numpy.array(cell[2])
                    pair_dist.append(numpy.linalg.norm(n_p))
                    molecule_positions.append(n_p)
                    molecule_symbols.append(symbols[jat])
                    if len(equivalent_indices) > 0:
                        eq_indices.append(equivalent_indices[jat])
    # Get reasonable dist_cut
    dist = pair_dist[:]
    dist.sort()
    if dist_cut_ > dist[1]:
        dist_cut = dist_cut_
    else:
        dist_cut = dist[12] + 0.7
        print(" The default dist_cut for extracting a centered cluster" +\
                "\n for symmetry evaluation = {}".format(dist_cut))
    molecule_positions = [mpos for mpos, pdist in
            zip(molecule_positions, pair_dist) if pdist < dist_cut]
    molecule_symbols = [msym for msym, pdist in
            zip(molecule_symbols, pair_dist) if pdist < dist_cut]
    eq_indices = [eind for eind, pdist in
            zip(eq_indices, pair_dist) if pdist < dist_cut]

    from itertools import groupby
    mol_name = ''
    mol_symbols = sorted(molecule_symbols)
    for key, group in groupby(mol_symbols):
        mol_name += key + str(len(list(group)))

    # local rotation if required.
    if locrot is not None:
        for i, pos in enumerate(molecule_positions):
            molecule_positions[i] = locrot.T.dot(pos)

    print(' molecule extracted {}:'.format(mol_name))
    print(" atom   x      y       z    distance")
    for symbo, position in zip(molecule_symbols, molecule_positions):
        print(" {:3s} {:6.2f} {:6.2f} {:6.2f}  {:8.4f}".format(
                *([symbo] + position.tolist() +
                [numpy.sqrt(numpy.dot(position, position))])))

    global imol
    with open('{}_{}.xyz'.format(mol_name, imol), 'w') as f:
        print(' {}\n'.format(len(molecule_symbols)), file=f)
        for symbo, position in zip(molecule_symbols, molecule_positions):
            print(" {:3s} {:6.2f} {:6.2f} {:6.2f}".format(
                    *([symbo] + position.tolist())), file=f)
    imol += 1
    return gmolecule(molecule_symbols, molecule_positions,
            equivalent_indices=eq_indices, fm_direction=fm_direction)


def xyz_get_rot_list(symbols, positions, tol=1.e-5, log=sys.stdout):
    '''
    Get the rotation element list of a molecule,
    given symboal and position list.
    '''
    mol = gmolecule(symbols, positions)
    return mol_get_rot_list(mol, tol=tol, log=log)


def mol_get_rot_list0(mol, tol=1.e-5, log=sys.stdout):
    from pymatgen.symmetry.analyzer import PointGroupAnalyzer,\
            generate_full_symmops
    from scipy.linalg import det

    analyzer = PointGroupAnalyzer(mol)
    print(" sch_symbol = {}".format(analyzer.sch_symbol), file=log)

    # Pick rotations only.
    symmops = [o for o in analyzer.symmops
            if abs(det(o.rotation_matrix)-1)<1.e-5]
    # symmops = generate_full_symmops(symmops, tol, max_recursion_depth=50)
    symmops = generate_full_symmops(symmops, tol)
    rot_list=[o.rotation_matrix for o in symmops]
    return rot_list


def chk_rot_keep(mol, rot, tol=1.e-5):
    if len(mol.equivalent_indices) > 1:
        coords = mol.cart_coords
        for i, coord in enumerate(coords):
            coordp = numpy.dot(rot, coord)
            diff = coords - coordp
            ind = numpy.where(numpy.all(numpy.abs(diff) < tol, axis=1))[0]
            if len(ind) != 1:
                print(' WARNING: identified rotation ids: {}'.format(ind))
                return False
            if mol.equivalent_indices[i] != mol.equivalent_indices[ind[0]]:
                return False
# not correct!
    if mol.fm_direction is not None:
        vec = numpy.dot(rot, mol.fm_direction)
# ok if in the same direction.
        if not numpy.allclose(vec, mol.fm_direction, rtol=tol):
            return False
    return True


def mol_get_rot_list_screening(mol, rot_list):
    '''Further screen rotations since atoms of the same symbol might be
    not equivalent, due to magnetism for instance.
    '''
    print(' number of rotations before screening: {}'.format(len(rot_list)))
    rot_list_tmp = rot_list
    rot_list = []
    numpy.set_printoptions(precision=3, suppress=True)
    for rot in rot_list_tmp:
        if chk_rot_keep(mol, rot):
            rot_list.append(rot)
        else:
            print(' rot \n{}\n removed!'.format(rot))
    print(' number of rotations after screening: {}'.format(len(rot_list)))
    return rot_list


def mol_get_rot_list(mol, tol=1.e-5, log=sys.stdout):
    rot_list = mol_get_rot_list0(mol, tol=tol, log=log)
    if len(mol.equivalent_indices) > 0 or mol.fm_direction is not None:
        rot_list = mol_get_rot_list_screening(mol, rot_list)
    return rot_list


if __name__ == "__main__":
    '''
    A test.
    '''
    # H cluster
    symbols = ['H', 'H', 'H', 'H', 'H', 'H']
    positions = [[0., 0., 0.], [2.0, 0., 0.], [0., 2.0, 0.0],
                 [0., -2.0, 0.], [0., 0., 2.0], [0., 0., -2.0]]
    rotations = xyz_get_rot_list(symbols, positions, log=sys.stdout)
    print(" H-dimer rotations:")
    print(rotations)
    # extract molecule
    symbols = ['Ce']
    scaled_positions = [[0, 0, 0]]
    cell = [[0.5, 0.5, 0.], [0.5, 0., 0.5], [0., 0.5, 0.5]]
    iat = 0
    dist_cut = -1.0
    molecule = xtal_extract_mol(symbols, scaled_positions, cell, iat, dist_cut)
    print(molecule.species)
    print(molecule.sites)



from __future__ import print_function
#Gutzwiller embedding Hamiltonian solver using machine learning.

import h5py, time, pickle, sys, numpy
import pygrisb.gsolver as gsolver


def get_input_soc_only(imp=1, l=3):
    '''get the embedding Hamiltonian parameters of impurity with index imp,
    given the orbital angular momentum l
    and assuming local spin-orbit interaction dominates.
    '''
    j2 = 2*l
    with h5py.File('EMBED_HAMIL_{}.h5'.format(imp), 'r') as f:
        d1 = f['/D'][0, 0]
        d2 = f['/D'][j2, j2]
        e1 = f['/H1E'][0, 0]
        e2 = f['/H1E'][j2, j2]
        l1 = f['/LAMBDA'][0,0]
        l2 = f['/LAMBDA'][j2, j2]
    mu = (e1+e2-l1-l2)/4.0
    e1 = e1 - mu
    e2 = e2 - mu
    l1 = l1 + mu
    l2 = l2 + mu
    return numpy.array([[d1, d2, e1, e2, l1, l2]]).real


def driver_gs_ml(l=3):
    '''driver for the machines learning based Gutzwiller solver.
    '''
    start_time = time.clock()

    imp = 1
    if '-i' in sys.argv:
        imp = int(sys.argv[sys.argv.index('-i')+1])
    if '-l' in sys.argv:
        l = int(sys.argv[sys.argv.index('-l')+1])
    if l == 3:
        subd = 'f_so_v0'
    else:
        raise ValueError(' l = {} not available!'.format(l))

    print(' solving emb. hamil. for impurity {}\n'.format(imp) +\
            ' with l = {} based on machine learning.'.format(l))

    dpath = gsolver.__path__[0]
    inp = get_input_soc_only(imp=imp, l=l)
    res = []
    for i in range(7):
        with open('{}/{}/kr_{}.pkl'.format(dpath, subd, i), \
                    'rb') as f:
            kr = pickle.load(f)
            res.append(kr.predict(inp))
    res = numpy.asarray(res)[:, 0]

    # density matrix
    na2 = (2*l+1)*2
    na4 = na2*2
    dm = numpy.zeros([na4, na4], dtype=numpy.complex)
    for i in range(2*l):
        dm[i, i] = res[0]
        dm[i + na2, i + na2] = res[2]
        dm[i, i + na2] = res[4]
        dm[i + na2, i] = numpy.conj(res[4])
    for i in range(2*l, na2):
        dm[i, i] = res[1]
        dm[i + na2, i + na2] = res[3]
        dm[i, i + na2] = res[5]
        dm[i + na2, i] = numpy.conj(res[5])

    with h5py.File('EMBED_HAMIL_RES_{}.h5'.format(imp), 'w') as f:
        f['/DM'] = dm.T
        f['/emol'] = res[-1:]

    end_time = time.clock()

    print('total time: {}'.format(end_time - start_time))
    sys.exit(0)


if __name__ == '__main__':
    driver_gs_ml()



'''Directly modify the Coulomb U-matrix of impurity
assuming Slater-Condon type parametrization.

Files needed:
    GPARAM.h5.
'''
import h5py


def get_v2e(u, j, imp):
    ryd2ev = 13.605698065894
    with h5py.File('GPARAM.h5', 'r') as f:
        iso = f['/iso'][0]
        # Upper-case Fortran convention to c-convention
        utrans = f['/IMPURITY_{}/DB_TO_SAB'.format(imp)][...].T

    # Get l quantum number
    l = (utrans.shape[0]/2-1)/2

    if iso == 2:
        from pygrisb.math.matrix_util import trans_JJ_to_CH_sup_sdn
        u_cmplx_harm_to_rel_harm = trans_JJ_to_CH_sup_sdn(l).T
        utrans = u_cmplx_harm_to_rel_harm.dot(utrans)

    from pygrisb.mbody.coulomb_matrix import U_matrix
    v2e, u_avg, j_avg = U_matrix('slater-condon', l, U_int=u,
            J_hund=j, T=utrans)
    print(' u_avg = {} j_avg = {}'.format(u_avg, j_avg))

    # CyGutz works with Rydberg units.
    v2e /= ryd2ev
    return v2e

if __name__ =='__main__':
    # impurity index
    imp = 1

    # U/J parameters to be specified (eV).
    u = 6.0
    j = 0.7

    v2e = get_v2e(u, j, imp)
    with h5py.File('EMBED_HAMIL_{}.h5'.format(imp)) as f:
        f['/V2E'][...] = v2e.T



from __future__ import print_function

# modiofied based on PythTB python tight binding module.

import numpy as np
from builtins import range, zip
from pygrisb.iface.wannierio import get_wannier_data



def tb_wigner_seitz(ngrid,lat):
    deg_ws = []
    rvec_ws = []
    ndiff = np.zeros(3)
    for n0 in range(-ngrid[0], ngrid[0]+1):
        for n1 in range(-ngrid[1], ngrid[1]+1):
            for n2 in range(-ngrid[2], ngrid[2]+1):
                dist_list = []
                for i0 in [0,1,2,-1,-2]:
                    ndiff[0] = n0 - i0*ngrid[0]
                    for i1 in [0,1,2,-1,-2]:
                        ndiff[1] = n1 - i1*ngrid[1]
                        for i2 in [0,1,2,-1,-2]:
                            ndiff[2] = n2 - i2*ngrid[2]
                            dist_list.append(np.linalg.norm(ndiff.dot(lat)))
                dist_list = np.asarray(dist_list)
                dist_min = np.min(dist_list)
                if np.abs(dist_list[0]-dist_min) < 1.e-7:
                    deg_ws.append(np.count_nonzero(\
                            np.abs(dist_list-dist_min) < 1.e-7))
                    rvec_ws.append(np.array([n0,n1,n2]))
    # sum-rule check
    deg_ws = np.array(deg_ws)
    rvec_ws = np.asarray(rvec_ws)
    tot1 = np.sum(1./deg_ws)
    tot2 = np.prod(ngrid)
    if np.abs(tot1 - tot2) > 1.e-7:
        raise ValueError("error in finding wigner-seitz points {} vs {}".\
                format(tot1, tot2))
    return deg_ws, rvec_ws


def get_tb_hr(kpoints,rpoints,wfwans,evals):
    phase_mat = np.exp(-2.j*np.pi*np.asarray(kpoints).dot(rpoints.T)) \
            /len(kpoints)
    hk_list = [[wfwansk1.T.conj().dot(np.diag(evalsk1)).dot(wfwansk1) \
            for wfwansk1, evalsk1 in zip(wfwans1,evals1)]\
            for wfwans1, evals1 in zip(wfwans,evals)]
    hk_list = np.array(hk_list).swapaxes(1,2).swapaxes(2,3)
    hr_list = np.tensordot(hk_list, phase_mat, axes=(3,0))
    return hr_list


class tb_model(object):
    r"""
    This is the main class of the PythTB package which contains all
    information for the tight-binding model.

    :param lat: Array containing lattice vectors in Cartesian
      coordinates (in arbitrary units). In example the below, the first
      lattice vector has coordinates [1.0,0.5] while the second
      one has coordinates [0.0,2.0].  By default, lattice vectors
      are an identity matrix.
    """

    def __init__(self,lat,deg_ws,rpoints,hr_list):
        self._dim_k = 3
        self._dim_r = 3
        self._lat=np.array(lat,dtype=float)
        if self._lat.shape != (self._dim_r,self._dim_r):
            raise Exception("\nWrong lat array dimensions")
        # check that volume is not zero and that have right handed system
        if np.abs(np.linalg.det(self._lat))<1.0E-6:
            raise Exception(\
                    "\nLattice vectors length/area/volume too"+\
                    " close to zero, or zero.")
        if np.linalg.det(self._lat)<0.0:
            raise Exception(\
                    "\n\nLattice vectors need to form right handed system.")
        self.deg_ws = np.asarray(deg_ws)
        self.rpoints = np.asarray(rpoints)
        self.hr_list = np.asarray(hr_list)
        self._norb = self.hr_list.shape[2]


    def _gen_ham(self,kpt,isp):
        """Generate Hamiltonian for a certain k-point,
        which is given in reduced coordinates!"""
        phase_mat = np.exp(-2.j*np.pi*self.rpoints.dot(kpt))/self.deg_ws
        ham = np.tensordot(self.hr_list[isp],phase_mat,axes=(2,0))
        return ham


    def _sol_ham(self,ham,eig_vectors=False):
        """Solves Hamiltonian and returns eigenvectors, eigenvalues"""
        # check that matrix is hermitian
        if np.max(ham-ham.T.conj())>1.0E-9:
            raise Exception("\n\nHamiltonian matrix is not hermitian?!")
        #solve matrix
        if eig_vectors==False: # only find eigenvalues
            eval = np.linalg.eigvalsh(ham)
            # sort eigenvalues and convert to real numbers
            eval = _nicefy_eig(eval)
            return np.array(eval,dtype=float)
        else: # find eigenvalues and eigenvectors
            eval,eig = np.linalg.eigh(ham)
            # sort evectors, eigenvalues and convert to real numbers
            eval,eig = _nicefy_eig(eval,eig)
            # reshape eigenvectors if doing a spinfull calculation
            return eval, eig

    def k_uniform_mesh(self,mesh_size):
        r"""
        Returns a uniform grid of k-points that can be passed to
        passed to function :func:`pythtb.tb_model.solve_all`.  This
        function is useful for plotting density of states histogram
        and similar.

        Returned uniform grid of k-points always contains the origin.

        :param mesh_size: Number of k-points in the mesh in each
          periodic direction of the model.

        :returns:

          * **k_vec** -- Array of k-vectors on the mesh that can be
            directly passed to function  :func:`pythtb.tb_model.solve_all`.

        Example usage::

          # returns a 10x20x30 mesh of a tight binding model
          # with three periodic directions
          k_vec = my_model.k_uniform_mesh([10,20,30])
          # solve model on the uniform mesh
          my_model.solve_all(k_vec)

        """

        # get the mesh size and checks for consistency
        use_mesh=np.array(list(map(round,mesh_size)),dtype=int)
        if use_mesh.shape!=(self._dim_k,):
            print(use_mesh.shape)
            raise Exception("\n\nIncorrect size of the specified k-mesh!")
        if np.min(use_mesh)<=0:
            raise Exception("\n\nMesh must have positive non-zero number of elements.")

        # construct the mesh
        if self._dim_k==1:
            # get a mesh
            k_vec=np.mgrid[0:use_mesh[0]]
            # normalize the mesh
            norm=np.tile(np.array(use_mesh,dtype=float),use_mesh)
            norm=norm.reshape(use_mesh.tolist()+[1])
            norm=norm.transpose([1,0])
            k_vec=k_vec/norm
            # final reshape
            k_vec=k_vec.transpose([1,0]).reshape([use_mesh[0],1])
        elif self._dim_k==2:
            # get a mesh
            k_vec=np.mgrid[0:use_mesh[0],0:use_mesh[1]]
            # normalize the mesh
            norm=np.tile(np.array(use_mesh,dtype=float),use_mesh)
            norm=norm.reshape(use_mesh.tolist()+[2])
            norm=norm.transpose([2,0,1])
            k_vec=k_vec/norm
            # final reshape
            k_vec=k_vec.transpose([1,2,0]).reshape([use_mesh[0]*use_mesh[1],2])
        elif self._dim_k==3:
            # get a mesh
            k_vec=np.mgrid[0:use_mesh[0],0:use_mesh[1],0:use_mesh[2]]
            # normalize the mesh
            norm=np.tile(np.array(use_mesh,dtype=float),use_mesh)
            norm=norm.reshape(use_mesh.tolist()+[3])
            norm=norm.transpose([3,0,1,2])
            k_vec=k_vec/norm
            # final reshape
            k_vec=k_vec.transpose([1,2,3,0]).reshape([use_mesh[0]*use_mesh[1]*use_mesh[2],3])
        else:
            raise Exception("\n\nUnsupported dim_k!")

        return k_vec


    def k_path(self,kpts,nk,report=True):
        r"""

        Interpolates a path in reciprocal space between specified
        k-points.  In 2D or 3D the k-path can consist of several
        straight segments connecting high-symmetry points ("nodes"),
        and the results can be used to plot the bands along this path.

        The interpolated path that is returned contains as
        equidistant k-points as possible.

        :param kpts: Array of k-vectors in reciprocal space between
          which interpolated path should be constructed. These
          k-vectors must be given in reduced coordinates.  As a
          special case, in 1D k-space kpts may be a string:

          * *"full"*  -- Implies  *[ 0.0, 0.5, 1.0]*  (full BZ)
          * *"fullc"* -- Implies  *[-0.5, 0.0, 0.5]*  (full BZ, centered)
          * *"half"*  -- Implies  *[ 0.0, 0.5]*  (half BZ)

        :param nk: Total number of k-points to be used in making the plot.

        :param report: Optional parameter specifying whether printout
          is desired (default is True).

        :returns:

          * **k_vec** -- Array of (nearly) equidistant interpolated
            k-points. The distance between the points is calculated in
            the Cartesian frame, however coordinates themselves are
            given in dimensionless reduced coordinates!  This is done
            so that this array can be directly passed to function
            :func:`pythtb.tb_model.solve_all`.

          * **k_dist** -- Array giving accumulated k-distance to each
            k-point in the path.  Unlike array *k_vec* this one has
            dimensions! (Units are defined here so that for an
            one-dimensional crystal with lattice constant equal to for
            example *10* the length of the Brillouin zone would equal
            *1/10=0.1*.  In other words factors of :math:`2\pi` are
            absorbed into *k*.) This array can be used to plot path in
            the k-space so that the distances between the k-points in
            the plot are exact.

          * **k_node** -- Array giving accumulated k-distance to each
            node on the path in Cartesian coordinates.  This array is
            typically used to plot nodes (typically special points) on
            the path in k-space.

        Example usage::

          # Construct a path connecting four nodal points in k-space
          # Path will contain 401 k-points, roughly equally spaced
          path = [[0.0, 0.0], [0.0, 0.5], [0.5, 0.5], [0.0, 0.0]]
          (k_vec,k_dist,k_node) = my_model.k_path(path,401)
          # solve for eigenvalues on that path
          evals = tb.solve_all(k_vec)
          # then use evals, k_dist, and k_node to plot bandstructure
          # (see examples)

        """

        # processing of special cases for kpts
        if kpts=='full':
            # full Brillouin zone for 1D case
            k_list=np.array([[0.],[0.5],[1.]])
        elif kpts=='fullc':
            # centered full Brillouin zone for 1D case
            k_list=np.array([[-0.5],[0.],[0.5]])
        elif kpts=='half':
            # half Brillouin zone for 1D case
            k_list=np.array([[0.],[0.5]])
        else:
            k_list=np.array(kpts)

        # in 1D case if path is specified as a vector, convert it to an (n,1) array
        if len(k_list.shape)==1 and self._dim_k==1:
            k_list=np.array([k_list]).T

        # make sure that k-points in the path have correct dimension
        if k_list.shape[1]!=self._dim_k:
            print('input k-space dimension is',k_list.shape[1])
            print('k-space dimension taken from model is',self._dim_k)
            raise Exception("\n\nk-space dimensions do not match")

        # must have more k-points in the path than number of nodes
        if nk<k_list.shape[0]:
            raise Exception("\n\nMust have more points in the path than number of nodes.")

        # number of nodes
        n_nodes=k_list.shape[0]

        # extract the lattice vectors from the TB model
        lat_per=np.copy(self._lat)
        # compute k_space metric tensor
        k_metric = np.linalg.inv(np.dot(lat_per,lat_per.T))

        # Find distances between nodes and set k_node, which is
        # accumulated distance since the start of the path
        #  initialize array k_node
        k_node=np.zeros(n_nodes,dtype=float)
        for n in range(1,n_nodes):
            dk = k_list[n]-k_list[n-1]
            dklen = np.sqrt(np.dot(dk,np.dot(k_metric,dk)))
            k_node[n]=k_node[n-1]+dklen

        # Find indices of nodes in interpolated list
        node_index=[0]
        for n in range(1,n_nodes-1):
            frac=k_node[n]/k_node[-1]
            node_index.append(int(round(frac*(nk-1))))
        node_index.append(nk-1)

        # initialize two arrays temporarily with zeros
        #   array giving accumulated k-distance to each k-point
        k_dist=np.zeros(nk,dtype=float)
        #   array listing the interpolated k-points
        k_vec=np.zeros((nk,self._dim_k),dtype=float)

        # go over all kpoints
        k_vec[0]=k_list[0]
        for n in range(1,n_nodes):
            n_i=node_index[n-1]
            n_f=node_index[n]
            kd_i=k_node[n-1]
            kd_f=k_node[n]
            k_i=k_list[n-1]
            k_f=k_list[n]
            for j in range(n_i,n_f+1):
                frac=float(j-n_i)/float(n_f-n_i)
                k_dist[j]=kd_i+frac*(kd_f-kd_i)
                k_vec[j]=k_i+frac*(k_f-k_i)

        if report==True:
            if self._dim_k==1:
                print(' Path in 1D BZ defined by nodes at '+str(k_list.flatten()))
            else:
                print('----- k_path report begin ----------')
                original=np.get_printoptions()
                np.set_printoptions(precision=5)
                print('real-space lattice vectors\n', lat_per)
                print('k-space metric tensor\n', k_metric)
                print('internal coordinates of nodes\n', k_list)
                if (lat_per.shape[0]==lat_per.shape[1]):
                    # lat_per is invertible
                    lat_per_inv=np.linalg.inv(lat_per).T
                    print('reciprocal-space lattice vectors\n', lat_per_inv)
                    # cartesian coordinates of nodes
                    kpts_cart=np.tensordot(k_list,lat_per_inv,axes=1)
                    print('cartesian coordinates of nodes\n',kpts_cart)
                print('list of segments:')
                for n in range(1,n_nodes):
                    dk=k_node[n]-k_node[n-1]
                    dk_str=_nice_float(dk,7,5)
                    print('  length = '+dk_str+'  from ',k_list[n-1],' to ',k_list[n])
                print('node distance list:', k_node)
                print('node index list:   ', np.array(node_index))
                np.set_printoptions(precision=original["precision"])
                print('----- k_path report end ------------')
            print()

        return (k_vec,k_dist,k_node)

def _nicefy_eig(eval,eig=None):
    "Sort eigenvaules and eigenvectors, if given, and convert to real numbers"
    # first take only real parts of the eigenvalues
    eval=np.array(eval.real,dtype=float)
    # sort energies
    args=eval.argsort()
    eval=eval[args]
    if not (eig is None):
        eig=eig[args]
        return (eval,eig)
    return eval


# for nice justified printout
def _nice_float(x,just,rnd):
    return str(round(x,rnd)).rjust(just)


def _nice_int(x,just):
    return str(x).rjust(just)


def _nice_complex(x,just,rnd):
    ret=""
    ret+=_nice_float(complex(x).real,just,rnd)
    if complex(x).imag<0.0:
        ret+=" - "
    else:
        ret+=" + "
    ret+=_nice_float(abs(complex(x).imag),just,rnd)
    ret+=" i"
    return ret


class w90(object):
    r"""

    This class of the PythTB package imports tight-binding model
    parameters from an output of a `Wannier90
    <http://www.wannier.org>`_ code.

    The `Wannier90 <http://www.wannier.org>`_ code is a
    post-processing tool that takes as an input electron wavefunctions
    and energies computed from first-principles using any of the
    following codes: Quantum-Espresso (PWscf), AbInit, SIESTA, FLEUR,
    Wien2k, VASP.  As an output Wannier90 will create files that
    contain parameters for a tight-binding model that exactly
    reproduces the first-principles calculated electron band
    structure.

    The interface from Wannier90 to PythTB will use only the following
    files created by Wannier90:

    - *prefix*.win
    - *prefix*\_hr.dat
    - *prefix*\_centres.xyz

    The first file (*prefix*.win) is an input file to Wannier90 itself. This
    file is needed so that PythTB can read in the unit cell vectors.

    To correctly create the second and the third file (*prefix*\_hr.dat and
    *prefix*\_centres.dat) one needs to include the following flags in the win
    file::

       hr_plot = True
       write_xyz = True
       translate_home_cell = False

    These lines ensure that *prefix*\_hr.dat and *prefix*\_centres.dat
    are written and that the centers of the Wannier functions written
    in the *prefix*\_centres.dat file are not translated to the home
    cell.  The *prefix*\_hr.dat file contains the onsite and hopping
    terms.

    The final two files (*prefix*\_band.kpt and *prefix*\_band.dat)
    are optional.  Please see documentation of function
    :func:`pythtb.w90.w90_bands_consistency` for more detail.

    So far we tested only Wannier90 version 2.0.1.

    .. warning:: For the time being PythTB is not optimized to be used
      with very large tight-binding models.  Therefore it is not
      advisable to use the interface to Wannier90 with large
      first-principles calculations that contain many k-points and/or
      electron bands.  One way to reduce the computational cost is to
      wannierize with Wannier90 only the bands of interest (for
      example, bands near the Fermi level).

    Units used throught this interface with Wannier90 are
    electron-volts (eV) and Angstroms.

    .. warning:: User needs to make sure that the Wannier functions
      computed using Wannier90 code are well localized.  Otherwise the
      tight-binding model might not interpolate well the band
      structure.  To ensure that the Wannier functions are well
      localized it is often enough to check that the total spread at
      the beginning of the minimization procedure (first total spread
      printed in .wout file) is not more than 20% larger than the
      total spread at the end of the minimization procedure.  If those
      spreads differ by much more than 20% user needs to specify
      better initial projection functions.

      In addition, please note that the interpolation is valid only
      within the frozen energy window of the disentanglement
      procedure.

    .. warning:: So far PythTB assumes that the position operator is
      diagonal in the tight-binding basis.  This is discussed in the
      :download:`notes on tight-binding formalism
      <misc/pythtb-formalism.pdf>` in Eq. 2.7.,
      :math:`\langle\phi_{{\bf R} i} \vert {\bf r} \vert \phi_{{\bf
      R}' j} \rangle = ({\bf R} + {\bf t}_j) \delta_{{\bf R} {\bf R}'}
      \delta_{ij}`.  However, this relation does not hold for Wannier
      functions!  Therefore, if you use tight-binding model derived
      from this class in computing Berry-like objects that involve
      position operator such as Berry phase or Berry flux, you would
      not get the same result as if you computed those objects
      directly from the first-principles code!  Nevertheless, this
      approximation does not affect other properties such as band
      structure dispersion.

    For the testing purposes user can download the following
    :download:`wannier90 output example
    <misc/wannier90_example.tar.gz>` and use the following
    :ref:`script <w90_quick>` to test the functionality of the interface to
    PythTB. Run the following command in unix terminal to decompress
    the tarball::

        tar -zxf wannier90_example.tar.gz

    and then run the following :ref:`script <w90_quick>` in the same
    folder.

    :param path: Relative path to the folder that contains Wannier90
       files.  These are *prefix*.win, *prefix*\_hr.dat,
       *prefix*\_centres.dat and optionally *prefix*\_band.kpt and
       *prefix*\_band.dat.

    :param prefix: This is the prefix used by Wannier90 code.
        Typically the input to the Wannier90 code is name *prefix*.win.

    Initially this function will read in the entire Wannier90 output.
    To create :class:`pythtb.tb_model` object user needs to call
    :func:`pythtb.w90.model`.

    Example usage::

      # reads Wannier90 from folder called *example_a*
      # it assumes that that folder contains files "silicon.win" and so on
      silicon=w90("example_a", "silicon")

    """

    def __init__(self,path="./",prefix="wannier"):
        # store path and prefix
        self.path=path
        self.prefix=prefix

        # read in lattice_vectors
        f=open(self.path+"/"+self.prefix+".win","r")
        ln=f.readlines()
        f.close()
        # get lattice vector
        self.lat=np.zeros((3,3),dtype=float)
        found=False
        for i in range(len(ln)):
            sp=ln[i].split()
            if len(sp)>=2:
                if sp[0].lower()=="begin" and sp[1].lower()=="unit_cell_cart":
                    # get units right
                    if ln[i+1].strip().lower()=="bohr":
                        pref=0.5291772108
                        skip=1
                    elif ln[i+1].strip().lower() in ["ang","angstrom"]:
                        pref=1.0
                        skip=1
                    else:
                        pref=1.0
                        skip=0
                    # now get vectors
                    for j in range(3):
                        sp=ln[i+skip+1+j].split()
                        for k in range(3):
                            self.lat[j,k]=float(sp[k])*pref
                    found=True
                    break
        if found==False:
            raise Exception( \
                    "Unable to find unit_cell_cart block in the .win file.")

        self.kgrid = None
        self.symbols = []
        self.atomic_positions = []
        read_position = 0
        for i,line in enumerate(ln):
            if "mp_grid" in line:
                line = line.split()
                self.kgrid = [int(x) for x in line[2:5]]
            elif "begin" in line and "atoms_cart" in line:
                read_position = 1
            elif "end" in line and "atoms_cart" in line:
                read_position = -1
            elif read_position > 0:
                if "bohr" in line.lower():
                    pref=0.5291772108
                elif "ang" in line.lower():
                    pref=1.0
                else:
                    line = line.split()
                    _symbol = line[0].split("_")[0]
                    symbol = _symbol[0:1].upper()
                    if len(_symbol) > 1:
                        symbol += _symbol[1:].lower()
                    self.symbols.append(symbol)
                    self.atomic_positions.append(
                            [float(x)*pref for x in line[1:4]])
            elif read_position < 0 and self.kgrid is not None:
                break
        if self.kgrid is None:
            raise Exception("Unable to find mp_grid!")
        # convert to scaled atomic position
        self.atomic_positions = np.asarray(self.atomic_positions).dot(\
                np.linalg.inv(self.lat))
        # get k-space data
        reals_lat, _, kpts, include_bands, wfwannier_list, bnd_es = \
                get_wannier_data(path=path)
        if not np.allclose(self.lat, reals_lat):
            raise ValueError("lattice vector inconsistent: {} vs {}"\
                    .format(self.lat, reals_lat))
        deg_ws, rpts = tb_wigner_seitz(self.kgrid, self.lat)
        hr_list = get_tb_hr(kpts, rpts, wfwannier_list, bnd_es)
        self.kpoints = kpts
        self.bnd_es = bnd_es
        self.hr_list = hr_list
        self.deg_ws = deg_ws
        self.rpoints = rpts


    def model(self):
        """
        This function returns :class:`pythtb.tb_model` object that can
        be used to interpolate the band structure at arbitrary
        k-point, analyze the wavefunction character, etc.
        """
        # make the model object
        return tb_model(self.lat,self.deg_ws,self.rpoints,self.hr_list)


def _cart_to_red(tmp,cart):
    "Convert cartesian vectors cart to reduced coordinates of a1,a2,a3 vectors"
    (a1,a2,a3)=tmp
    # matrix with lattice vectors
    cnv=np.array([a1,a2,a3])
    # transpose a matrix
    cnv=cnv.T
    # invert a matrix
    cnv=np.linalg.inv(cnv)
    # reduced coordinates
    red=np.zeros_like(cart,dtype=float)
    for i in range(0,len(cart)):
        red[i]=np.dot(cnv,cart[i])
    return red

def _red_to_cart(tmp,red):
    "Convert reduced to cartesian vectors."
    (a1,a2,a3)=tmp
    # cartesian coordinates
    cart=np.zeros_like(red,dtype=float)
    for i in range(0,len(cart)):
        cart[i,:]=a1*red[i][0]+a2*red[i][1]+a3*red[i][2]
    return cart



if __name__ == "__main__":
    wannier90 = w90()
    wmodel = wannier90.model()
    kpt = wannier90.kpoints[25]
    print(kpt)
    print(wannier90.bnd_es[0][25])
    ham = wmodel._gen_ham(kpt,isp=0)
    evals = wmodel._sol_ham(ham)
    print(evals)



'''
Multiplets analysis for LDA+G+SOC calculations.
'''

import numpy as np
import h5py
from pygrisb.mbody.multiplets_analysis_lib import get_local_histogram


def calc_save_atomic_states(imp=1, num_ev=100, fname='multiplets.h5'):
    '''
    Calculate and save the eigen-values of the local many-body density matrix
    and the labels.
    '''
    # Get eigen-values and labels of the local atomic states.
    vals, n_labels, s_labels, l_labels, j_labels, \
            chi_labels, multiplet_degeneracies, \
            eval_s, eval_l, eval_j, ent_entropy \
            = get_local_histogram(imp, num_ev=num_ev)

    # Store results to metadata.
    with h5py.File(fname, 'a') as f:
        base = "/impurity_" + str(imp)
        if base in f:
            del f[base]
        f[base + "/sum_weights"] = np.sum(vals)
        f[base + "/weights"] = vals
        f[base + "/n_labels"] = n_labels
        f[base + "/s_labels"] = s_labels
        f[base + "/l_labels"] = l_labels
        f[base + "/j_labels"] = j_labels
        f[base + "/eval_s"] = eval_s
        f[base + "/eval_l"] = eval_l
        f[base + "/eval_j"] = eval_j
        f[base + "/ent_entropy"] = ent_entropy
        f[base + "/deg_labels"] = multiplet_degeneracies


def plot_atomic_states(imp=1, num_label=5):
    '''
    Plot the atomic state probabilities using the formula \rho = EXp(-F)
    '''
    # Read in metadata
    with h5py.File("multiplets.h5", 'r') as f:
        base = "/impurity_" + str(imp)
        vals = f[base + "/weights"][()]
        multiplet_degeneracies = f[base + "/deg_labels"][()]
        n_labels = f[base + "/n_labels"][()]
        s_labels = f[base + "/s_labels"][()]

    # Generate data corresponding to \rho = exp(-F)
    f_wt = -np.log(vals / multiplet_degeneracies)
    f_wt = f_wt - f_wt[0]

    # Labels of the atomic states
    labels = ['N=%1d,S=%3.1f' % (int(n + 0.1), j) for n, j in
              zip(n_labels[:num_label], s_labels[:num_label])]

    # Scatter plot with annotation.
    import matplotlib.pyplot as plt
    plt.scatter(f_wt, vals)
    for label, x, y in zip(labels, f_wt[:num_label], vals[:num_label]):
        plt.annotate(
            label,
            xy=(x, y), xytext=(0, 20),
            textcoords='offset points', ha='center', va='bottom',
            bbox=dict(boxstyle='round,pad=0.5', fc='yellow', alpha=0.5),
            arrowprops=dict(arrowstyle='->', connectionstyle='arc3,rad=0'))
    plt.xlim(-0.1, 12)
    plt.ylim(0, 1)
    plt.xlabel("$f_{n}$")
    plt.ylabel("$e^{-f_{n}}d_{n}/\sum_{n}{e^{-f_{n}}d_{n}}$")
    plt.title("Eigen-values of the local many-body density matrix")

    plt.show()


if __name__ == "__main__":
    calc_save_atomic_states(imp=1, num_ev=400)
    plot_atomic_states(imp=1, num_label=7)



from __future__ import print_function
try:
    from builtins import range
except:
    pass

import h5py, numpy


def get_full_fock_states_by_int(norb):
    '''generate all the fock states represented by integers.
    '''
    fock_list = [[] for i in range(norb+1)]
    for i in range(2**norb):
        n1 = bin(i).count('1')
        fock_list[n1].append(i)
    return fock_list


def get_double_fock_states_at_half_filling(fock_list):
    '''generate the impurity-bath fock states at half-filling.
    '''
    norb = len(fock_list)-1
    base = 2**norb
    nstates = sum([len(fs)**2 for fs in fock_list])
    fock2_array = numpy.empty(nstates, dtype=numpy.int32)
    iadd = 0
    for i, fs in enumerate(fock_list):
        for fs1 in fs:
            for fs2 in fock_list[norb-i]:
                fock2_array[iadd] = fs1 + fs2*base
                iadd += 1
    return fock2_array


def h5add_fock_states(norb, f_h5):
    '''add the fock state basis.
    '''
    fock_list = get_full_fock_states_by_int(norb)
    fock2_array = get_double_fock_states_at_half_filling(fock_list)
    f_h5["/fock_basis/dim"] = [len(fock2_array)]
    f_h5["/fock_basis/vals"] = fock2_array


if __name__ == "__main__":
    with h5py.File("fock.h5", "w") as f:
        h5add_fock_states(14, f)



from __future__ import print_function
import numpy
from pygrisb.estructure.gwannier import get_symkpath


kpath = get_symkpath()
print("lattice")
print(kpath._structure.lattice.matrix)
print(kpath._prim.lattice.matrix)

print("reciprocal lattice")
print(kpath._structure.lattice.reciprocal_lattice.matrix)
print(kpath._prim.lattice.reciprocal_lattice.matrix)

print("k_path")
print(kpath.kpath)



import numpy as np

data_u1 = np.loadtxt('mu_u3.5.dat').T
data_u2 = np.loadtxt('mu_u5.0.dat').T


import matplotlib.pyplot as plt
plt.figure(1)
plt.plot(data_u1[0], data_u1[1], label='u=3.5')
plt.plot(data_u2[0], data_u2[1], label='u=5.0')
plt.legend()
plt.ylabel('$E_{tot}$')
plt.xlabel('$\mu$')
plt.figure(2)
plt.plot(data_u1[0], data_u1[2], label='u=3.5')
plt.plot(data_u2[0], data_u2[2], label='u=5.0')
plt.legend()
plt.ylabel('$n_f$')
plt.xlabel('$\mu$')
plt.show()



"""
Some commonly-used functions of matrix.
"""
import numpy as np
import scipy.sparse as sps
import scipy.linalg as slinalg
from scipy.sparse import csr_matrix,csc_matrix
from itertools import product

def get_random_hermitian_matrix(n):
    '''
    Generate random hermitian matrix(n, n)
    '''
    x = np.random.rand(n, n) + 1.j * np.random.rand(n, n)
    return x + np.conj(x.T)


def get_random_unitary_matrix(n):
    '''
    Generate random unitary matrix.
    '''
    x = get_random_hermitian_matrix(n)
    w, v = np.linalg.eigh(x)
    return v


def get_matrices_bk_list(matrices, rtol=1.e-12):
    '''
    Get the block dimensions of a group of matrices.
    '''
    l = matrices[0].shape[0]
    l_list = []
    lbase = 0
    for i in range(l):
        is_bk = True
        if i < l - 1:
            for matrix in matrices:
                if np.amax(np.abs(matrix[lbase : i + 1, i + 1 :])) > rtol:
                    is_bk = False
                    break
                if np.amax(np.abs(matrix[i + 1 :, lbase : i + 1])) > rtol:
                    is_bk = False
                    break
        if is_bk:
            l_list.append(i + 1 - lbase)
            lbase = i + 1
    return l_list


def trans_orbital_fast_to_spin_fast(mat, lback=False):
    '''
    Convert a matrix from orbital-fast basis to spin-fast basis or backward
    if lback = True.
    '''
    dim = len(mat)
    U = np.zeros((dim, dim), dtype=int)  # <orbital-fast | spin fast>
    i = 0
    for j in range(0, dim, 2):
        U[i][j] = 1
        i += 1
    for j in range(1, dim, 2):
        U[i][j] = 1
        i += 1
    if lback:
        return np.dot(U, np.dot(mat, U.T))
    else:
        return np.dot(U.T, np.dot(mat, U))


def Rtrans_spin_block(mat):
    '''
    Convert a matrix from spin-down-spin-up block to spin-up-spin-down block
    or backward.
    '''
    dim = len(mat)
    U = np.zeros((dim, dim), dtype=int)
    i = 0
    for j in range(dim / 2, dim):
        U[i][j] = 1
        i += 1
    for j in range(0, dim / 2):
        U[i][j] = 1
        i += 1
    return np.dot(mat, U)


def get_random_unitary_matrix_spin_deg(n, spin_fast=True):
    '''
    Generate spin-degenerate random unitary matrix in the spin-faster basis.
    '''
    u = get_random_unitary_matrix(n)
    from scipy.linalg import block_diag
    u2 = block_diag(u, u)
    if spin_fast:
        u2 = trans_orbital_fast_to_spin_fast(u2)
    return u2


def set_eigen_space(vals):
    '''
    Identify the eigen-space using eigen-values vals.
    '''
    sec_id = [0]
    num_vals = len(vals)
    for i in range(1, num_vals):
        if abs((vals[i] - vals[i - 1]) / max(vals[i - 1], 1.e-10)) > 1.e-4:
            sec_id.append(i)
    sec_id.append(num_vals)
    return sec_id


def sym_array(X, R_list):
    '''
    Symmetrize matrix X.
    '''
    Ham = np.zeros_like(X)
    for i, R in enumerate(R_list):
        if isinstance(R, np.ndarray):
            Ham += R.dot(X).dot(R.conj().T)
        elif isinstance(R, csr_matrix):
            Ham += R.dot(X).dot(R.todense().conj().T)
        else:
            raise AssertionError("Unsupported data type of {}!".\
                    format(R.__class__))
    return Ham / len(R_list)


def sym_csr_matrix(X, R_list):
    '''
    Symmetrize csr_matrix X.
    '''
    from scipy.sparse import csr_matrix
    X_sym = csr_matrix(X.shape, dtype=X.dtype)
    for i, R in enumerate(R_list):
        X_sym += R * X * R.getH()
    return X_sym / len(R_list)


def shrink_label(labels, sec_id, method="average"):
    '''
    Shrink labels for eigen-space.
    method = 'average' ot 'sum'.
    '''
    if labels is None:
        return None
    s_labels = []
    for i in range(len(sec_id) - 1):
        if sec_id[i + 1] <= sec_id[i]:
            continue
        tmp = np.sum(labels[sec_id[i]:sec_id[i + 1]])
        if method == "average":
            tmp /= sec_id[i + 1] - sec_id[i]
        s_labels.append(tmp)
    return np.array(s_labels)


def Rtrans_U_supdn_to_spin_fast_sdnup(U):
    '''
    U = U * untary_{orbital fast (up, down)} {spin fast (down, up)}
    '''
    dim = len(U)
    V = np.zeros((dim, dim), dtype=int)
    i = 0
    for j in range(1, dim, 2):
        V[i][j] = 1
        i += 1
    for j in range(0, dim, 2):
        V[i][j] = 1
        i += 1
    return np.dot(U, V)


def get_utrans_orbfast_supdn_to_spin_fast_supdn(n):
    '''
    get a unitary transformation from orbital fast spin up-down
    basis to spin fast spin up-down basis.
    '''
    v = np.zeros((n, n), dtype=int)
    i = 0

    # spin up block
    for j in range(0, n, 2):
        v[i][j] = 1
        i += 1

    # spin down block
    for j in range(1, n, 2):
        v[i][j] = 1
        i += 1
    return v


def trans_JJ_to_CH_sup_sdn(L):
    '''
    trafoso provides transformation matrices from
    |L,1/2,mL,mS> (L=0,1,2,3, mS=-1/2,1/2) basis to
    basis |J,L,S,mJ>, J=L-1/2, L+1/2
    H. Watanabe 'Operator Methods in Ligand Field Theory'
    Prentice Hall, 1966, Table 1.8-1.
    ordering because of the convention used in WIEN is:
                       mS=1/2        mS=-1/2
                     -L .......L  -L ...... L     (2*(2L+1) columns)
            -(L-1/2)
               .
    J=L-1/2    .
               .
             (L-1/2)
             -L-1/2
               .
    J=L+1/2    .
               .
              L+1/2
    '''
    cf = np.zeros((2 * (2 * L + 1), 2 * (2 * L + 1)))
    if L == 0:
        cf[0, 1] = 1.0
        cf[1, 0] = 1.0
    else:
        k1 = -1
        for ms in range(-1, 2, 2):
            ams = -ms / 2.
            for ml in range(-L, L + 1):
                k1 = k1 + 1
                k2 = -1
                for mj in range(-2 * L + 1, 2 * L, 2):  # L-1/2 states
                    amj = mj / 2.
                    k2 = k2 + 1
                    d = amj - ml - ams
                    if abs(d) < 0.0001:
                        if ms == 1:
                            cf[k2, k1] = - \
                                np.sqrt((L + 0.5 + amj) / (2 * L + 1))
                        else:
                            cf[k2, k1] = np.sqrt((L + 0.5 - amj) / (2 * L + 1))
                for mj in range(-2 * L - 1, 2 * L + 2, 2):  # L+1/2 states
                    amj = mj / 2.
                    k2 = k2 + 1
                    d = amj - ml - ams
                    if abs(d) < 0.0001:
                        if ms == 1:
                            cf[k2, k1] = np.sqrt((L + 0.5 - amj) / (2 * L + 1))
                        else:
                            cf[k2, k1] = np.sqrt((L + 0.5 + amj) / (2 * L + 1))
    return cf


def matrix_basis_expand(mat, basis):
    '''
    Expand mat in basis.
    '''
    res = np.zeros_like(mat)
    for b in basis:
        res += np.vdot(b, mat) * b
    return res


def get_loewner_matrix_ev(e_vals, f, fp):
    '''
    Loewner matrix, ref: positive definite matrices, Bhatia, p.154.
    f: function; fp: first derivative.
    '''
    loewner = []
    for i, ei in enumerate(e_vals):
        _loewner = []
        for j, ej in enumerate(e_vals):
            if np.abs(ei - ej) < 1.e-10:
                res = fp(ei)
            else:
                res = (f(ei) - f(ej))/(ei - ej)
            _loewner.append(res)
        loewner.append(_loewner)
    return np.array(loewner)


def yield_derivative_f_matrix(a, h_list, f, fp):
    '''
    a = \sum_{c} {c*h_list}. fp(x) = \par f(x) / \par x
    Yields \par f(a) / \par c.
    '''
    w, v = np.linalg.eigh(a)
    vherm = np.conj(v.T)
    loewner = get_loewner_matrix_ev(w, f, fp)
    for h in h_list:
        _h = np.dot(vherm, np.dot(h, v))
        yield np.dot(v, np.dot(loewner*_h, vherm))


def get_func_matrix_eigh(A, func, alpha=1.0):
    '''
    Compute matrix function of alpha*A.

    Parameters
    ----------
    A : (N, N) array_like or sparse matrix
        Matrix to be exponentiated.
    alpha : float or complex, optional
        Scale factor. Default is 1.0
    func : string.
        Type of function. Should be one of

            - 'exp' - exponential function.
            - 'log' - natural logarithm.
            - 'sqrt' - square root.
            - 'abs' - absolute value function.

        Otherwise UndefinedFunction exception will be raised.

    Returns
    -------
    res : (N, N) ndarray
        Matrix function of `alpha*A`.
    '''
    if A.shape == (1, 1):
        vals, vecs = np.array([A[0,0]]), np.array([[1.0]], dtype=np.complex)
    else:
        if isinstance(A, csr_matrix) or isinstance(A, csc_matrix):
            A = A.todense()
        vals, vecs = np.linalg.eigh(A)
    if func == 'exp':
        res = np.dot(np.dot(vecs,np.diag(np.exp(alpha * vals))),
                np.conj(vecs).T)
    elif func == 'log':
        # regularize if needed.
        _min = np.min(vals)
        if _min < 1.e-20:
            vals += 1.e-20-_min
        res = np.dot(np.dot(vecs,np.diag(np.log(alpha * vals))),
                np.conj(vecs).T)
    elif func == 'sqrt':
        # regularize if needed.
        _min = np.min(vals)
        if _min < 1.e-20:
            vals += 1.e-20-_min
        res = np.dot(np.dot(vecs,np.diag(np.sqrt(alpha * vals))),
                np.conj(vecs).T)
    elif func == 'abs':
        res = np.dot(np.dot(vecs,np.diag(np.abs(alpha * vals))),
                np.conj(vecs).T)
    else:
        raise Exception('UndefinedFunction')
    return res


def get_matrix_trace(A):
    if A.shape == (1, 1):
        tr = A[0,0]
    elif A.shape == (1,):
        tr = A[0]
    else:
        if sps.issparse(A):
            tr = np.sum(A.diagonal())
        else:
            tr = np.trace(A)
    return tr


def get_loewdin_orthnorm(a):
    '''Get the Loewdin symmetry-adapted orthonormalization via
    singular-value decomposition.
    '''
    U, s, V = np.linalg.svd(a)
    return U.dot(V)


def unitary_transform_coulomb_matrix(a, u):
    '''Perform a unitary transformation (u) on the Coulomb matrix (a).
    '''
    a_ = np.asarray(a).copy()
    m_range = range(a.shape[0])
    for i,j in product(m_range, m_range):
        a_[i,j,:,:] = u.T.conj().dot(a_[i,j,:,:].dot(u))
    a_ = a_.swapaxes(0,2).swapaxes(1,3)
    for i,j in product(m_range, m_range):
        a_[i,j,:,:] = u.T.conj().dot(a_[i,j,:,:].dot(u))
    return a_


def get_hamilt_matrix_from_ev(w, v):
    '''given v_{alpha, n} and w_{n}, get back the Hamiltonian matrix.
    Here alpha is the basis orbital index and n the band index.
    '''
    vh = v.T.conj()
    for i in range(v.shape[0]):
        v[i] *= w
    return v.dot(vh)


def solve_ab_plus_ba_equal_c(b, c):
    '''solve for a given a*b + b*a = c and b is hermitian.
    '''
    err = np.max(np.abs(b-b.T.conj()))
    if err > 1.e-8:
        raise AssertionError("b not hermitian with error = {}!".format(err))
    w,v = np.linalg.eigh(b)
    cp = v.T.conj().dot(c).dot(v)
    ap = [[cp[i,j]/(w[i]+w[j]) for j in range(b.shape[0])] \
            for i in range(b.shape[0])]
    a = v.dot(ap).dot(v.T.conj())
    return a


def get_1stderivative_sroot_entropyn_at_h(a, h):
    '''
    evaluate p_{\squareroot(a(1-a))} / p_h.
    '''
    k0 = a.dot(np.eye(a.shape[0])-a)
    x0 = slinalg.sqrtm(k0)
    k1 = h - a.dot(h) - h.dot(a)
    res = solve_ab_plus_ba_equal_c(x0, k1)
    return res


def get_2ndderivative_sroot_entropyn_at_h(a, h1, h2):
    '''
    evaluate p2_{\squareroot(a(1-a))} / p_h1 / p_h2.
    '''
    p1 = get_1stderivative_sroot_entropyn_at_h(a, h1)
    p2 = get_1stderivative_sroot_entropyn_at_h(a, h2)
    k2 = -h1.dot(h2) - h2.dot(h1)
    c = k2 - (p1.dot(p2) + p2.dot(p1))
    k0 = a.dot(np.eye(a.shape[0])-a)
    x0 = slinalg.sqrtm(k0)
    res = solve_ab_plus_ba_equal_c(x0, c)
    return res




import h5py, numpy


def compare_array(path, fname1='GLOG.h5', fname2='GLOG_REF.h5'):
    with h5py.File(fname1, 'r') as f:
        data1 = f[path][()]
    with h5py.File(fname2, 'r') as f:
        data2 = f[path][()]
    return numpy.allclose(data1, data2)




import numpy

'''Special models.
'''


class semicircular(object):
    '''semi-circular DOS.
    '''
    def __init__(self):
        '''define dos and cumulative dos function.
        '''
        self.dos = lambda e: 2./numpy.pi * numpy.sqrt(1-e**2)
        self.cdos = lambda e: (e*numpy.sqrt(1-e**2) \
                + numpy.arcsin(e)) / numpy.pi + 0.5

    def get_e_list_of_uniform_wt(self, nmesh=5000):
        '''Get the energy mesh with uniform weight.
        '''
        cdos_list = numpy.linspace(0,1,nmesh+1)
        from scipy.optimize import bisect
        e_list = [bisect(lambda x: self.cdos(x)-a, -1 ,1) \
                for a in cdos_list]
        e_list = numpy.asarray(e_list)
        e_list = (e_list[1:] + e_list[0:-1])/2
        return e_list



from __future__ import print_function

import sys
from pygrisb.gutz.init_magnet_conf import init_magnet_conf

def initialize():
    '''
    Initialization for the Gutzwiller-Slave-Boson solver.
    Store all the relevant information in GPARAM.5 file.
    '''
    log_file = open("init_ga.slog", 'w')
    print(" The executed commant line is:", file=log_file)
    print("    "+" ".join(sys.argv[:]), file=log_file)

    from pygrisb.gutz.structure import get_gatoms, check_material
    from pygrisb.gutz.gatom import h5calc_save_lrot
    # get the material
    material = get_gatoms()
    material.set_modify_mode()

    # Q & A
    from pygrisb.gutz.usrqa import h5save_usr_qa_setup
    h5save_usr_qa_setup(material, log=log_file)

    # Setting the attributes to material
    material.h5set()

    # Set the self-energy structure
    material.set_SelfEnergy()

    # Set the Lie parameters for 3d-rotations
    material.set_LieParameters()

    # Set one-particle rotation matrix list
    material.set_one_particle_rotation_list()

    # Set Coulomb matrix list
    material.set_v2e_list()

    # Set S/L matrix vector in symmetry-adapted basis
    material.set_SL_vector_list()

    # check some info in log
    check_material(material, log_file)

    # save_gparam.h5
    from pygrisb.gutz.ginput import save_gparam
    save_gparam(iso=material.iso, ispin=material.ispin,
            ityp_list=material.ityp_list,
            imap_list=material.imap_list, na2_list=material.na2_list,
            imix=material.gimix, iembeddiag=material.giembeddiag,
            sigma_list=material.sigma_list, v2e_list=material.v2e_list,
            sx_list=material.sx_list, sy_list=material.sy_list,
            sz_list=material.sz_list, lx_list=material.lx_list,
            ly_list=material.ly_list, lz_list=material.lz_list,
            utrans_list=material.utrans_list, ldc=material.ldc,
            u_avg_list=material.u_avg_list,
            j_avg_list=material.j_avg_list,
            nelf_list=material.nelf_list,
            rotations_list=material.rotations_list,
            lie_odd_params_list=material.Lie_Jodd_list,
            lie_even_params_list=material.Lie_Jeven_list,
            jgenerator_list=material.jgenerator_list,
            sp_rotations_list=material.sp_rotations_list,
            nval_bot_list=material.nval_bot_list,
            nval_top_list=material.nval_top_list)

    # local rotation matrix (l) in Hilbert space.
    h5calc_save_lrot(material)
    log_file.close()


if __name__ == '__main__':
    initialize()



# Author: Yongxin Yao

'''One band with semi-circular dos.
'''

import numpy as np
import pygrisb.gutz.ginput as ginput
import pygrisb.model.tbASE as tb
import pygrisb.model.special as special


def gutz_model_setup(u=0.0, nmesh=5000, mu=0):
    '''Set up Gutzwiller calculations for 1-band model with semi-circular DOS.

    Parameters:

    * u: real number
      Hubbard U.
    * nmesh: interger number
      number of energy mesh
    * mu: real number
      chemical potential

    Result:

    Create all the necessary input file of ``GPARAMBANDS.h5``, ``GPARAM.h5``,
    and ``BAREHAM_0.h5``, for *CyGutz* calculation.
    '''
    # get semi-circular class, predifined in pygrisb.model.special
    sc = special.semicircular()

    # get semi-circular energy mesh
    e_list = sc.get_e_list_of_uniform_wt(nmesh=nmesh)

    # get atomic structure with 1 atom per unit cell.
    a = tb.AtomsTB("N", [(0, 0, 0)], cell=(1, 1, 1))

    # specify single s-orbital and orbital dimensions.
    a.set_orbitals_spindeg(orbitals=[('s')])
    norb = 1
    norb2 = norb*2

    # get a list of one-body Hamilonian for the enrgy mesh.
    hk_list = [np.array([[e+0.j]]) for e in e_list]

    # get the tight-binding model
    aTB = tb.TB(a, hk_list=hk_list)

    # here the energy mesh is the same as k-points.
    kps = e_list
    num_k = len(kps)

    # set uniform k-point weight.
    kps_wt = 1.0 / num_k * np.ones((num_k))

    # Include the spin degeneracy to k-point weight.
    if aTB.Atoms.spindeg:
        kps_wt *= 2

    # We will work with grand canonical model, num_e will not be used.
    num_e = 0.0

    # list of one-body part of the local Hamiltonians.
    # here the s-orbital is located at zero.
    h1e_list = [np.zeros((norb2, norb2), dtype=np.complex)]

    # generate GPARAMBANDS.h5
    # ensemble is set to 1 for grand canonical system.
    ginput.save_gparambands(kps_wt, num_e, norb, \
            ensemble=1, h1e_list=h1e_list, delta=1.e-8)

    # set self-energy structure
    sigma = np.zeros((norb2,norb2), dtype=int)
    sigma[0::2, 0::2] = np.arange(1,norb**2+1).reshape( \
            (norb,norb))
    sigma[1::2, 1::2] = sigma[0::2, 0::2]

    # set Coulomb matrix
    v2e = np.zeros((norb2,norb2,norb2,norb2), dtype=np.complex)
    v2e[0,0,0,0] = v2e[0,0,1,1] = v2e[1,1,0,0] = v2e[1,1,1,1] = u

    # set the potential shift such that the system has particle-hole
    # symmetry with recpect to zero.
    # also include chemical potential here.
    vdc2_list = np.array([np.zeros((norb2))])
    vdc2_list[0, 0:2] = -u/2 + mu

    # generate GPARAM.h5 file.
    ginput.save_gparam(na2_list=[norb2],
            iembeddiag=-1, imix=0, sigma_list=[sigma],
            v2e_list=[v2e], nval_top_list=[norb2],
            vdc2_list=vdc2_list, max_iter=500)

    # generate BAREHAM_0.h5 file.
    aTB.save_bareham(kps)




from __future__ import print_function
'''
Help functions for io.
'''
try:
    from builtins import range
except:
    pass

import numpy as np
import h5py, glob


def read_slice(lines):
    n = len(lines) / 2
    U = []
    for i in range(n):
        V = []
        line_real = lines[i]
        line_real = line_real.split()
        line_imag = lines[i + n]
        line_imag = line_imag.split()
        for j in range(n):
            V.append(float(line_real[j]) + float(line_imag[j]) * 1.j)
        U.append(V)
    U = np.array(U).T  # Fortran convension to C convension
    return U


def read_TRANS(fileName):
    '''
    Read transformation matrix.
    '''
    with open(fileName, 'r') as f:
        lines = f.readlines()
    line = lines[0]
    line = line.split()
    numOrbitals = int(line[0])
    numIons = int(line[1])
    u_list = []
    iBase = 1
    for ion in range(numIons):
        u_list.append(read_slice(lines[iBase: iBase + numOrbitals * 2]))
        iBase += numOrbitals * 2
    return u_list


def read_Hs(fileName):
    '''
    Read Hermitian matrix basis.
    '''
    with open(fileName, 'r') as f:
        lines = f.readlines()
    line = lines[0]
    line = line.split()
    numIons = int(line[0])
    iLine = 1
    HsAll = []
    for ion in range(numIons):
        Hs = []
        line = lines[iLine]
        line = line.split()
        numOrbitals = int(line[0])
        numHs = int(line[1])
        iLine += 1
        for iHs in range(numHs):
            Hs.append(read_slice(lines[iLine: iLine + numOrbitals * 2]))
            iLine += numOrbitals * 2
        HsAll.append(Hs)
    return HsAll


def write_slice(f, U):
    for V in U.T:
        f.write("".join("%20.16f" % (x.real) for x in V) + '\n')
    for V in U.T:
        f.write("".join("%20.16f" % (x.imag) for x in V) + '\n')


def write_TRANS(fileName, u_list):
    '''
    Write transformation matrix.
    '''
    numIons = len(u_list)
    numOrbitals = len(u_list[0])
    f = open(fileName, 'w')
    f.write(" %5i %5i" % (numOrbitals, numIons) + '\n')
    for u in u_list:
        write_slice(f, u)
    f.close()


def write_RLNEF(fileName, RAll, LA1All, NKSAll, EFermi):
    '''
    Write new solution under the current basis transformation.
    '''
    numIons = len(RAll)
    numOrbitals = len(RAll[0])
    with open(fileName, 'w') as f:
        f.write(" %5i %5i" % (numOrbitals, numIons) + '\n')
        for ion in range(numIons):
            write_slice(f, RAll[ion])
            write_slice(f, LA1All[ion])
            write_slice(f, NKSAll[ion])
        f.write(" %20.12f" % (EFermi))


def write_Hs(fileName, HsAll):
    '''
    Write new solution under the current basis transformation.
    '''
    numIons = len(HsAll)
    with open(fileName, 'w') as f:
        f.write(" %5i" % (numIons) + '\n')
        for ion in range(numIons):
            numHs = len(HsAll[ion])
            f.write(" %5i %5i" % (len(HsAll[ion][0]), numHs) + '\n')
            for iHs in range(numHs):
                write_slice(f, HsAll[ion][iHs])


def write_sigma_struct(sigma_list, file_name="WH_SIGMA_STRUCT.INP"):
    '''
    Write WH_SIGMA_STRUCT.INP file.
    '''
    with open(file_name, 'w') as f:
        for ni, sigma in enumerate(sigma_list):
            f.write(" %5i %5i" % (ni + 1, len(sigma)) + '\n')
            for col in np.array(sigma).T:
                f.write(' '.join('%4d' % (x) for x in col) + '\n')


def write_frozen(ne_mott_list, orb_mott_list):
    '''
    Write FROZEN.INP file.
    '''
    with open("FROZEN.INP", 'w') as f:
        for ni, orbs in enumerate(orb_mott_list):
            f.write(" %5i ! NI" % (ni + 1) + '\n')
            f.write(" %5i %5i ! N_ORB, N_ELECTRON_MOTT_LOCALIZED" % (
                    len(orbs), ne_mott_list[ni]) + '\n')
            if len(orbs) > 0:
                f.write(''.join(" %5i " % (orb + 1) for orb in orbs) + '\n')


def write_WH_SL_VEC(s_vec_list, l_vec_list):
    '''
    Write WH_SL_VEC.INP file.
    '''
    s_vec_list = np.swapaxes(s_vec_list, 0, 1)
    l_vec_list = np.swapaxes(l_vec_list, 0, 1)
    with open("WH_SL_VEC.INP", 'w') as f:
        for _s_list in s_vec_list:
            for _s in _s_list:
                write_slice(f, _s)
        for _l_list in l_vec_list:
            for _l in _l_list:
                write_slice(f, _l)


def special_reorder(a_list):
    if len(a_list) != 1 or a_list[0].shape[0] != 6:
        print('Dimension not correct, special reorder cancelled!')
        return a_list

    umat = np.array([
        [0, 1, 0],
        [1, 0, 0],
        [0, 0, 1]], dtype=complex)

    mat2 = np.zeros((6, 6), dtype=complex)
    mat2[0:3, 0:3] = umat.dot(a_list[0].T[::2, ::2]).dot(umat.T.conj()).T
    mat2[3:6, 3:6] = mat2[0:3, 0:3]
    return [mat2]


def h52txt_rln(fh5_name, ftxt_name, specialt=False):
    lma_list = []
    nks_list = []
    r_list = []
    with h5py.File(fh5_name, 'r') as f:
        for imp in range(1000):
            path = '/IMPURITY_{}'.format(imp+1)
            if path in f:
                lma_list.append(f[path+'/LAMBDA'][()].T)
                nks_list.append(f[path+'/NKS'][()].T)
                r_list.append(f[path+'/R'][()].T)
            else:
                break
    if specialt:
        lma_list = special_reorder(lma_list)
        nks_list = special_reorder(nks_list)
        r_list = special_reorder(r_list)

    write_RLNEF(ftxt_name, r_list, lma_list, nks_list, 0.0)


def h52txt_rln_list(h5p='WH_RL_INIT.h5', txtp='WH_RLNEF.INP', specialt=False):
    for fname in glob.glob(h5p+'*'):
        h52txt_rln(fname, txtp+fname[len(h5p):], specialt=specialt)



if __name__=='__main__':
    h52txt_rln_list(specialt=True)



from __future__ import print_function
import numpy as np
import sys, os, glob, subprocess, shutil, fileinput, h5py
import pygrisb.basic.splot as splot
import pygrisb.basic.units as units
from pygrisb.run import gwien


'''help routines for processing wien2k data. Assume the current work directory
has a subfolder template with reference case.struct and possibly case.inso.
'''


def get_rotations(case_fname):
    '''get the rotation list in struct file.
    '''
    with open(str(case_fname), 'r') as f:
        lines = f.readlines()
        for i, line in enumerate(lines):
            if 'SYMMETRY OPERATIONS' in line:
                break
        nops = int(lines[i].split()[0])
        lines = lines[i+1:]
        rot_list = []
        for i in range(0,nops*4,4):
            rot = []
            lok = True
            for j in range(3):
                line = lines[i+j]
                rot.append([int(line[:2]), int(line[2:4]), int(line[4:6])])
                if abs(float(line[6:17])) > 1.e-6:
                    lok = False
                    break
            if lok:
                if abs(np.linalg.det(rot)-1)<1.e-6:
                    rot_list.append(rot)
    return rot_list


def get_equivalent_atom_indices(case_fname):
    '''get equivalent atomic indices according to the atom labels in
    case.struct file.
    '''
    with open(str(case_fname), 'r') as f:
        idx_equivalent_atoms = []
        ibase = 0
        for line in f:
            if 'MULT=' in line:
                mult = int(line.split()[1])
                for i in range(mult):
                    idx_equivalent_atoms.append(ibase)
                ibase += mult
    return idx_equivalent_atoms


def get_local_rotations(case_fname):
    '''get list of locrot in struct file.
    '''
    with open(str(case_fname), 'r') as f:
        locrot_list = []
        readline = 100
        for line in f:
            if 'MULT=' in line:
                mult = int(line.split()[1])
            elif 'LOCAL ROT MATRIX' in line:
                readline = 0
                locrot = []
            if readline < 3:
                locrot.append(map(float, [line[20:30], line[30:40], \
                        line[40:50]]))
                readline += 1
                if readline == 3:
                    # wien2k convention
                    locrot = np.asarray(locrot).T
                    for i in range(mult):
                        # maybe there are problems here.
                        locrot_list.append(locrot)
    return locrot_list


def get_volume(file_scf='./case.scf'):
    '''Get primitive unit volume from scf file f_scf.
    '''
    with open(file_scf, 'r') as f:
        for line in f:
            if ':VOL' in line:
                line = line.split('=')
                vol = float(line[1])
        return vol


def get_scf_energy(file_scf='./case.scf'):
    '''Get primitive unit volume from scf file f_scf.
    '''
    with open(file_scf, 'r') as f:
        for line in f:
            if ':ENE' in line:
                line = line.split('=')
                etot = float(line[1])
        return etot


def create_struct(dir_template, vfrac, case='case'):
    '''Create a case.struct file with volume of vfrac of the reference
    struct file in dir_template.
    '''
    f_struct = glob.glob(dir_template+'/*struct')
    if len(f_struct) == 0:
        raise IOError('No struct file existing in {}!'.format(dir_template))
    f_struct = f_struct[0]
    with open(f_struct, 'r') as fs:
        with open(case+'.struct', 'w') as fd:
            for i, line in enumerate(fs.readlines()):
                if i == 3:
                    a_list = []
                    for j in range(3):
                        a_list.append(float(line[j*10:(j+1)*10]))
                    a_list = np.array(a_list)
                    a_list *= vfrac**(1./3)
                    line = '{:10.6f}{:10.6f}{:10.6f}'.format(*a_list) + \
                            line[30:]
                fd.write(line)


def update_case_ref(vfrac_min):
    '''Update case.struct based on the initial run of the Vmin sample.
    '''
    cwd = os.getcwd()+'/'
    os.chdir('./template')
    create_struct('../Ref_Vmin/case', 1./vfrac_min, case='case')
    os.chdir(cwd)


def create_dir1(vfrac, dname, dir_template='./template'):
    '''Create a subdirectory dname in the current directory with
    volumes changing by a vfrac wth repect to the reference structure
    in the dir_template.
    '''
    # create it if necessary
    if not os.path.isdir(dname):
        os.mkdir(dname)
    if not os.path.isdir(dname+'/case'):
        os.mkdir(dname+'/case')

    cwd = os.getcwd()+'/'
    # go to the subdirectory
    os.chdir(dname+'/case')

    # cretae the case.struct file
    create_struct(cwd+dir_template, vfrac)

    # return to the upper work directory
    os.chdir(cwd)


def create_dir_list(vfrac_min=0.7, vfrac_max=1.3, vfrac_step=0.05, \
        dir_template='./template'):
    '''Create a list of subdirectories in the current directory with
    volumes changing by a fraction of vfrac_step in the range of
    [vfrac_min, vfrac_max] wth repect to the reference structure
    in the dir_template.
    '''

    if not os.path.isdir(dir_template):
        raise IOError('{} does not exist!'.format(dir_template))
    # Get volume fraction list
    vfrac_list = np.arange(vfrac_min, vfrac_max+1.e-5, vfrac_step)

    # loop over to create subdirectory list
    f_log = open('vol_record.txt', 'w')
    for i, vfrac in enumerate(vfrac_list):

        # subdirectory name
        dname = 'V{}'.format(i)
        f_log.write('{}  {}\n'.format(dname, vfrac))

        create_dir1(vfrac, dname, dir_template=dir_template)


def get_dirvlist():
    dir_list = glob.glob('V*')
    for i,v in enumerate(dir_list):
        dir_list[i] = v.split('V')[1]
    dir_list.sort()
    for i,v in enumerate(dir_list):
        dir_list[i] = 'V' + str(v)
    return dir_list


def batch_init_lapw(args=['-vxc', '5', '-rkmax', '8.5', '-numk', '5000']):
    '''Loop over all the directories to run init_lapw.
    '''
    f = open('binit_lapw.log', 'w')
    cmd = [os.environ['WIENROOT']+'/init_lapw', '-b'] + args
    cwd = os.getcwd()+'/'
    for dname in get_dirvlist():
        os.chdir(dname+'/case')
        subprocess.call(cmd, stdout=f, shell=True)
        os.chdir(cwd)
    f.close()


def modify_emax_case_in1(case_in1='case.in1', emax=7.5):
    '''Modify the emax value in case_in1 for the case with
    spin-orbit calculation.
    '''
    for line in fileinput.input(files=case_in1, inplace=True):
        if 'emax' in line:
            line = line.replace(line[33:38], '{:5.1f}'.format(emax))
        print(line, end='')


def batch_initso_lapw(dir_template='./template', emax=7.5):
    '''Loop over all the directories to run initso_lapw -- actually,
    because there is no batch mode provided, it simply copy the case.inso
    from dir_template and modify the EMAX value to 7.5 in case.in1.
    '''
    cwd = os.getcwd()+'/'
    for dname in get_dirvlist():
        os.chdir(dname+'/case')
        shutil.copy(cwd+'/'+dir_template+'/case.inso', './')
        shutil.copy(cwd+'/'+dir_template+'/case.in2c', './')
        modify_emax_case_in1(case_in1='case.in1', emax=emax)
        os.chdir(cwd)


def h2get_energy_volume_list(path='lapw'):
    '''Loop over all the directories to get the energy vs volume data,
    and save it to the metadata file results.h5 at path.
    '''
    v_list = []
    e_list = []
    cwd = os.getcwd()+'/'
    for dname in get_dirvlist():
        os.chdir(dname+'/case/'+path)
        v_list.append(get_volume())
        e_list.append(get_scf_energy())
        os.chdir(cwd)

    # Ryd/Bohr units to eV/A units
    v_list = np.array(v_list)*units.Bohr_A**3
    e_list = np.array(e_list)*units.Ryd_eV

    with h5py.File('results.h5', 'a') as f:
        if not 'vol_list' in f:
            f['/vol_list'] = v_list
        epath = '/'+path+'/etot_list'
        if epath in f:
            del f[epath]
        f[epath] = e_list
    splot.xy_plot(v_list, e_list, xlabel='V ($\AA^{3}$/primitive cell)',
            ylabel='E (eV/primitive cell)', fsave='ev_{}.pdf'.format(path))


def compare_ev_plot(path_list, fres='results.h5'):
    '''Compare lapw/lapwso/lapwsog in a plot.
    '''
    v_list = []
    e_list = []
    pattern_list = []
    label_list = []
    with h5py.File(fres, 'r') as f:
        postfix=''
        for path in path_list:
            if path in f:
                label_list.append(path)
                e_list.append(f['/'+path+'/etot_list'][()])
                v_list.append(f['/vol_list'][()])
                pattern_list.append('o')
                postfix += path.split('/')[0]
        if v_list != []:
            splot.xy2_plot(v_list, e_list, pattern_list, label_list,
                    xlabel='V ($\AA^{3}$/primitive cell)',
                    ylabel='E (eV/primitive cell)',
                    fsave='ev_{}.pdf'.format(postfix))


def compare_pv_plot(path_list, fres='results.h5'):
    '''Compare lapw/lapwso/lapwsog in a plot.
    '''
    v_list = []
    p_list = []
    pattern_list = []
    label_list = []
    with h5py.File(fres, 'r') as f:
        postfix=''
        for path in path_list:
            if path in f:
                label_list.append(path)
                p_list.append(f['/'+path+'/eosfit/p_list'][()])
                v_list.append(f['/'+path+'/eosfit/v_list'][()])
                pattern_list.append('o')
                postfix += path.split('/')[0]
        if v_list != []:
            splot.xy2_plot(v_list, p_list, pattern_list, label_list,
                    xlabel='V ($\AA^{3}$/primitive cell)',
                    ylabel='P (GPa)',
                    fsave='pv_{}.pdf'.format(postfix))


def batch_save_lapw(sdir='lapw', args=['-f']):
    '''Loop over all the directories to save_lapw.
    '''
    cmd = [os.environ['WIENROOT']+'/save_lapw', '-d'] + [sdir] + args
    cwd = os.getcwd()+'/'
    for dname in get_dirvlist():
        os.chdir(dname+'/case')
        subprocess.call(cmd, shell=True)
        os.chdir(cwd)


def run_lapw(args=['-i', '70'], nproc=1):
    '''Loop over all the directories to run_lapw using nproc processors
    with arguments provided in args.
    '''
    cmd = [os.environ['WIENROOT']+'/run_lapw'] + args
    cwd = os.getcwd()+'/'
    if '-p' in sys.argv:
        nproc = int(sys.argv[sys.argv.index('-p')+1])
    for i,dname in enumerate(get_dirvlist()):
        os.chdir(dname+'/case')
        proc = subprocess.Popen(cmd)
        os.chdir(cwd)
        if (i+1) % nproc == 0:
            proc.communicate()


def batch_copy_dir(src, dst):
    '''Loop over all the directories and copy subfolder src to dst.
    '''
    cwd = os.getcwd()+'/'
    for dname in get_dirvlist():
        os.chdir(dname+'/case')
        if not os.path.lexists(dst):
            os.mkdir(dst)
        for f in os.listdir(src):
            shutil.copy(src+'/'+f, dst)
        os.chdir(cwd)


def steps(vfrac_min=0.7, vfrac_max=1.3, vfrac_step=0.05, \
        dir_template='./template'):
    if len(sys.argv) == 1 or '-h' in sys.argv[1]:
        print('Please provide with inline argument chosen from below:\n' +
                '  Vmin -- setup init. calculation of the min. vol. point;\n' +
                '  update_case_ref -- update esp. RMT values of ref. case;\n' +
                '  Vlist -- generate directories for a range of volumes;\n' +
                '  batch_init_lapw -- init_lapw all the directories;\n' +
                '  batch_initso_lapw -- initso_lapw all the directories;\n' +
                '  batch_init_ga -- init ga for all the directories;\n' +
                '  batch_init_mott -- init ga-mott for all directories;\n' +
                '  batch_setu_x_j_x -- set u/j for all directories;\n' +
                '  batch_modify_ga ... -- modify ga for all directories;\n' +
                '  batch_run_lapw -- run_lapw all the directories 1 by 1;\n' +
                '  batch_run_lapwso -- run_lapw -so all the directories;\n' +
                '  batch_run_ga -- run_ga all the directories; \n' +
                '  batch_save_lapw -- save_lapw all the directories; \n' +
                '  batch_save_lapwso -- save_lapwso all the directories; \n' +
                '  batch_gsave_uxjx -- save DFT+G results to uxjx dirs; \n' +
                '  batch_gsave_uxjx -- save DFT+G results to uxjx dirs; \n' +
                '  -cp-dir src dst -- copy subfilder src to dst; \n' +
                '  -jobq u j -w dir -- batch submit jobs; \n' +
                '  -ev lapw -- save_energy volume data for lapw calc.; \n' +
                '  -ev lapwso -- save e-v data for lapwso calc.\n' +
                '  -ev dirname -- save e-v data for dirname calc.\n' +
                '  -pv lapw -- Murnaghan EOS fir for lapw results;\n' +
                '  -pv lapwso -- Murnaghan EOS fir for lapwso results.\n' +
                '  -pv uxjx -- Murnaghan EOS fir for DFT+G results.\n' +
                '  -pv dirname -- Murnaghan EOS fir for dirname results.\n' +
                '  -ev-cmp path1 path2 ... -- Compare e-v data.\n' +
                '  -pv-cmp path1 path2 ... -- Compare p-v data.\n' +
                '  You may append "-p nproc" to specify # of procs in use.')
        sys.exit('Please choose proper inline argument!')
    if 'Vmin' in sys.argv[1]:
        create_dir1(vfrac_min, 'Ref_Vmin', dir_template=dir_template)
        print('Please goto Ref_Vmin/case and finish manual test.')
    elif 'update_case_ref' == sys.argv[1]:
        update_case_ref(vfrac_min)
    elif 'Vlist' == sys.argv[1]:
        create_dir_list(vfrac_min=vfrac_min, vfrac_max=vfrac_max,
                vfrac_step=vfrac_step, dir_template=dir_template)
    elif 'batch_init_lapw' == sys.argv[1]:
        batch_init_lapw()
    elif 'batch_run_lapw' == sys.argv[1]:
        run_lapw()
    elif 'batch_run_lapwso' == sys.argv[1]:
        run_lapw(args=['-so', '-i', '70'])
    elif 'batch_save' in sys.argv[1]:
        batch_save_lapw(sdir=sys.argv[1].split('_')[2])
    elif 'batch_gsave' in sys.argv[1]:
        gwien.batch_gsave(sdir=sys.argv[1].split('_')[2])
    elif '-cp-dir' == sys.argv[1]:
        batch_copy_dir(sys.argv[2], sys.argv[3])
    elif '-ev' == sys.argv[1]:
        h2get_energy_volume_list(path=sys.argv[2])
    elif 'batch_initso_lapw' == sys.argv[1]:
        batch_initso_lapw()
    elif '-pv' == sys.argv[1]:
        from pygrisb.dft.eos import h5get_mfit_ev
        h5get_mfit_ev(path=sys.argv[2])
    elif '-ev-cmp' == sys.argv[1]:
        compare_ev_plot(sys.argv[2:])
    elif '-pv-cmp' == sys.argv[1]:
        compare_pv_plot(sys.argv[2:])
    elif 'batch_init_ga' == sys.argv[1]:
        gwien.batch_init_ga()
    elif 'batch_init_mott' == sys.argv[1]:
        gwien.batch_init_mott()
    elif 'batch_setu' in sys.argv[1]:
        _args = sys.argv[1].split('_')
        args = ['-unique_u_ev', _args[2], '-unique_j_ev', _args[4]]
        gwien.batch_modify_ga_setup(args)
    elif 'batch_modify_ga' == sys.argv[1]:
        args = sys.argv[2:]
        gwien.batch_modify_ga_setup(args)
    elif '-jobq' == sys.argv[1]:
        gwien.batch_job_slurm(sys.argv[2], sys.argv[3])
    elif 'batch_run_ga' == sys.argv[1]:
        gwien.run_ga()
    else:
        raise ValueError('Inline option not defined!')



if __name__=='__main__':
    steps(vfrac_step=0.05)



from __future__ import print_function

import unittest
import numpy as np
from scipy.linalg import expm,sqrtm
from pygrisb.math.matrix_util import yield_derivative_f_matrix, \
        get_1stderivative_sroot_entropyn_at_h, \
        get_2ndderivative_sroot_entropyn_at_h


class KnowValues(unittest.TestCase):
    def test_derivative_exp_ix(self):
        # random Hermitian matrix of dimention 5.
        n = 5
        x = np.random.rand(n, n) + np.random.rand(n, n)*1.j
        x += np.conj(x.T)
        # Numerically calculate the derivative w.r.t. h.
        h = np.array([ \
            [0,              0, 1./np.sqrt(2.), 0, 0],
            [0,              0, 0,              0, 0],
            [1./np.sqrt(2.), 0, 0,              0, 0],
            [0,              0, 0,              0, 0],
            [0,              0, 0,              0, 0]])

        expix0 = expm(-1.j*x)
        delta = 1.e-7
        expix1 = expm(-1.j*(x + delta*h))
        partial0 = (expix1 - expix0)/delta
        f = lambda x: np.exp(-1.j*x)
        fp = lambda x: -1.j*np.exp(-1.j*x)
        partials = yield_derivative_f_matrix(x, [h], f, fp)
        for partial in partials:
            err = np.abs(np.max(partial - partial0))
            self.assertAlmostEqual(err, 0.)

    def test_derivative_srootn(self):
        # random Hermitian matrix of dimention 5.
        n = 5
        x = np.random.rand(n, n) + np.random.rand(n, n)*1.j
        x += np.conj(x.T)
        _,v = np.linalg.eigh(x)
        w = np.random.rand(n)
        # get a valid density matrix
        x = v.dot(np.diag(w)).dot(v.T.conj())

        # Numerically calculate the derivative w.r.t. h.
        h = np.array([ \
            [0,              0, 1./np.sqrt(2.), 0, 0],
            [0,              0, 0,              0, 0],
            [1./np.sqrt(2.), 0, 0,              0, 0],
            [0,              0, 0,              0, 0],
            [0,              0, 0,              0, 0]])
        delta = 1.e-7
        xp = x + delta*h
        sroot0 = sqrtm((np.eye(x.shape[0])-x).dot(x))
        sroot1 = sqrtm((np.eye(x.shape[0])-xp).dot(xp))
        partial0 = (sroot1 - sroot0)/delta
        partial1 = get_1stderivative_sroot_entropyn_at_h(x, h)

        f = lambda x: np.sqrt((1-x)*x)
        fp = lambda x: 0.5/np.sqrt((1-x)*x)*(1-2*x)
        partials = yield_derivative_f_matrix(x, [h], f, fp)

        for partial2 in partials:
            err = np.abs(np.max(partial2 - partial0))
            self.assertAlmostEqual(err, 0., places=6)
        err = np.abs(np.max(partial1 - partial0))
        self.assertAlmostEqual(err, 0., places=6)


    def test_2ndderivative_srootn(self):
        # random Hermitian matrix of dimention 5.
        n = 5
        x = np.random.rand(n, n) + np.random.rand(n, n)*1.j
        x += np.conj(x.T)
        _,v = np.linalg.eigh(x)
        w = np.random.rand(n)
        # get a valid density matrix
        x = v.dot(np.diag(w)).dot(v.T.conj())

        # Numerically calculate the derivative w.r.t. h.
        h1 = np.array([ \
            [0,              0, 1./np.sqrt(2.), 0, 0],
            [0,              0, 0,              0, 0],
            [1./np.sqrt(2.), 0, 0,              0, 0],
            [0,              0, 0,              0, 0],
            [0,              0, 0,              0, 0]])
        h2 = np.array([ \
            [0,              0, 0, 1./np.sqrt(2.), 0],
            [0,              0, 0,              0, 0],
            [0,              0, 0,              0, 0],
            [1./np.sqrt(2.), 0, 0,              0, 0],
            [0,              0, 0,              0, 0]])
        delta = 1.e-7
        xp = x + delta*h2
        p0 = get_1stderivative_sroot_entropyn_at_h(x, h1)
        p1 = get_1stderivative_sroot_entropyn_at_h(xp, h1)
        partial0 = (p1 - p0)/delta

        partial1 = get_2ndderivative_sroot_entropyn_at_h(x, h1, h2)
        err = np.abs(np.max(partial1 - partial0))
        self.assertAlmostEqual(err, 0., places=6)


    def test_derivative_tr_exp_ix_a(self):
        # random Hermitian matrix of dimention 5.
        n = 5
        a = np.random.rand(n, n)
        x = np.random.rand(n, n) + np.random.rand(n, n)*1.j
        x += np.conj(x.T)
        # Numerically calculate the derivative w.r.t. h.
        h = np.array([ \
            [0,              0, 1./np.sqrt(2.), 0, 0],
            [0,              0, 0,              0, 0],
            [1./np.sqrt(2.), 0, 0,              0, 0],
            [0,              0, 0,              0, 0],
            [0,              0, 0,              0, 0]])

        expix0 = expm(-1.j*x)
        delta = 1.e-7
        expix1 = expm(-1.j*(x + delta*h))
        partial0 = (np.trace(np.dot(expix1, a)) -
                np.trace(np.dot(expix0, a)))/delta
        f = lambda x: np.exp(-1.j*x)
        fp = lambda x: -1.j*np.exp(-1.j*x)
        partials = yield_derivative_f_matrix(x, [h], f, fp)
        for partial in partials:
            # trace
            _partial = np.sum(partial.T*a)
            err = np.abs(_partial - partial0)
            self.assertAlmostEqual(err, 0.)


if __name__=="__main__":
    print("Tests for matrix_util.")
    unittest.main()



import numpy as np
from pygrisb.model import circauxi
import shutil,subprocess

u_list = np.arange(5.0, 5.1, 0.1)
cmd = ['/home/ykent/WIEN_GUTZ/bin2/CyGutz', '-r', '-1']
mu=2.1
for u in u_list:
    circauxi.gutz_model_setup(u=u, nmesh=5000, norb=3, tiny=0.0, mu=mu)
    subprocess.call(cmd)
    shutil.copyfile('WH_RL_BEST.h5', 'WH_RL_INIT.h5')
    shutil.copyfile('WH_RL_BEST.h5', 'WH_RL_INIT.h5_u{}mu{}'.format(u, mu))




import numpy as np
import pygrisb.gutz.ginput as ginput
import pygrisb.model.tbASE as tb
import pygrisb.model.special as special


def get_hk1(e, tiny, norb, mu):
    _diag = [e+0.j+mu]
    for i in range(1,norb):
        _diag.append(e*tiny)
    return np.diag(_diag)


def gutz_model_setup(u=0.0, nmesh=5000, norb=1, tiny=0.0, mu=0):
    '''Set up calculations for semi-circular DOS with two-ghost orbitals.
    '''
    # semi-circular energy mesh
    sc = special.semicirular()
    e_list = sc.get_e_list_of_uniform_wt(nmesh=nmesh)

    a = tb.AtomsTB("N", [(0, 0, 0)], cell=(1, 1, 1))
    a.set_orbitals_spindeg(orbitals=[tuple(["s" for i in range(norb)])])

    # set up hk_list
    hk_list = [get_hk1(e, tiny, norb, mu) for e in e_list]
    aTB = tb.TB(a, hk_list=hk_list)

    # dummy k-point list
    kps = e_list
    num_k = len(kps)
    kps_wt = 1.0 / num_k * np.ones((num_k))
    if aTB.Atoms.spindeg:
        kps_wt *= 2
    num_e = 0.0
    norb2 = norb*2

    # GPARAMBANDS.h5

    h1e_list = [np.zeros((norb2, norb2), dtype=np.complex)]
    ginput.save_gparambands(kps_wt, num_e, norb, \
            ensemble=1, h1e_list=h1e_list)

    # GPARAM.h5
    # self-energy structure
    sigma = np.zeros((norb2,norb2), dtype=int)
    sigma[0::2, 0::2] = np.arange(1,norb**2+1).reshape( \
            (norb,norb))
    sigma[1::2, 1::2] = sigma[0::2, 0::2]

    # Coulomb matrix
    v2e = np.zeros((norb2,norb2,norb2,norb2), dtype=np.complex)
    v2e[0,0,0,0] = v2e[0,0,1,1] = v2e[1,1,0,0] = v2e[1,1,1,1] = u
    vdc2_list = np.array([np.zeros((norb2))])
    vdc2_list[0, 0:2] = -u/2

    ginput.save_gparam(na2_list=[norb2],
            iembeddiag=-3, imix=0, sigma_list=[sigma],
            v2e_list=[v2e], nval_top_list=[norb2],
            vdc2_list=vdc2_list)

    # BAREHAM_0.h5
    aTB.save_bareham(kps)


if __name__=='__main__':
    gutz_model_setup(u=2.50, nmesh=5000, norb=3, tiny=0.0)



#!/usr/bin/env python
from __future__ import print_function
import numpy,h5py
from pygrisb.gutz.usrqa import get_usr_input


def init_subval_tsolver():
    '''setup the parameters for sub-valence truncation solver.
    '''
    with h5py.File('GPARAM.h5', 'r') as f:
        imap_imp = f['/IMAP_IMP'][()]
        gpms_list = []
        ngpm_list = []
        nval_list = []
        ngp_list = []
        for i, imap in enumerate(imap_imp):
            # one-base fortran index
            imp = i+1
            if imap < imp:
                gpms_list.append(gpms_list[imap])
                ngpm_list.append(ngpm_list[imap])
                nval_list.append(nval_list[imap])
                ngp_list.append(ngp_list[imap])
            elif imap == imp:
                sx = f['/IMPURITY_{}/SX'.format(imp)][()].T
                sy = f['/IMPURITY_{}/SY'.format(imp)][()].T
                sz = f['/IMPURITY_{}/SZ'.format(imp)][()].T
                lx = f['/IMPURITY_{}/LX'.format(imp)][()].T
                ly = f['/IMPURITY_{}/LY'.format(imp)][()].T
                lz = f['/IMPURITY_{}/LZ'.format(imp)][()].T
                jx, jy, jz = sx + lx, sy + ly, sz + lz
                j2 = jx.dot(jx) + jy.dot(jy) + jz.dot(jz)
                # assuming 1-l channel
                l = (sx.shape[0]/2-1)/2
                ngpm = numpy.zeros([2], dtype=numpy.int)
                gpms = numpy.zeros([2, 2*l+2], dtype=numpy.int)
                j21, j22 = (l-0.5)*(l+0.5), (l+0.5)*(l+1.5)
                for ii, val in enumerate(j2.diagonal()):
                    if abs(val - j21) < 1.e-6:
                        # for frotran, one-base
                        gpms[0, ngpm[0]] = ii + 1
                        ngpm[0] += 1
                    elif abs(val - j22) < 1.e-6:
                        # for frotran, one-base
                        gpms[1, ngpm[1]] = ii + 1
                        ngpm[1] += 1
                    else:
                        raise ValueError( \
                                ' inconsistent setup with j2-diagonal = {}'\
                                .format(val))
                print(' IMPURITY {}\n num orbs groups = {}'.format(imp, 2))
                nvals = []
                for ig in range(2):
                    print(' group {} member indices: {}'.format(ig, \
                            gpms[ig, :ngpm[ig]]))
                    nval_bot = get_usr_input( \
                            ' choose min. occupancy of physical subspace', \
                            map(str, range(ngpm[ig]+1)))
                    nval_bot = int(nval_bot)
                    nval_top = get_usr_input( \
                            ' choose min. occupancy of physical subspace', \
                            map(str, range(nval_bot, ngpm[ig]+1)))
                    nval_top = int(nval_top)
                    nvals.append([nval_bot, nval_top])
                    nval_bot = get_usr_input( \
                            ' choose min. occupancy of variational subspace', \
                            map(str, range(ngpm[ig]+1)))
                    nval_bot = int(nval_bot)
                    nval_top = get_usr_input( \
                            ' choose min. occupancy of variational subspace', \
                            map(str, range(nval_bot, ngpm[ig]+1)))
                    nval_top = int(nval_top)
                    nvals.append([nval_bot, nval_top])
                nval_list.append(nvals)
                ngpm_list.append(ngpm)
                gpms_list.append(gpms)
                ngp_list.append(2)
            else:
                raise ValueError(' imap = {} > imp = {}'.format(imap, imp))

    with h5py.File('GESOLVER.h5', 'w') as f:
        for i, ngp in enumerate(ngp_list):
            imp = i + 1
            f['/IMPURITY_{}/n_orbs_group'.format(imp)] = [ngp]
            f['/IMPURITY_{}/n_gpmembers'.format(imp)] = ngpm_list[i]
            f['/IMPURITY_{}/gpmember_list'.format(imp)] = gpms_list[i]
            f['/IMPURITY_{}/gpnocc_range'.format(imp)] = nval_list[i]


if __name__ == '__main__':
    init_subval_tsolver()



import os
import glob
import warnings


def get_wiencase():
    # Determine WIEN2k case name
    case = os.path.basename(os.getcwd())
    # directory name is not case (happens when submitting to cluster)
    if not os.path.isfile(case + '.struct'):
        files = glob.glob('*.struct')
        if len(files) < 1:
            raise Exception('No struct file present.')
        elif len(files) > 1:
            # heuristic algorithm to determine case:
            # the struct file whose head is the same as the most files in this
            # directory is most likely the case
            candidates = [os.path.splitext(f)[0] for f in files]
            allheads = [os.path.splitext(f)[0] for f in os.listdir('.')]
            counts = [len([1 for f in allheads if f == candidate])
                      for candidate in candidates]
            index = counts.index(max(counts))
            case = candidates[index]
            warnings.warn(
                " case heuristically selected from multiple possibilities!")
        else:
            case, ext = os.path.splitext(os.path.basename(files[0]))
    return case



import numpy as np
import h5py
import sys
from pygrisb.math.matrix_basis import sigmatomatrixbasis


def save_gparam(iso=1, ispin=1, ityp_list=[0], imap_list=[0], na2_list=[2],
        imix=0, amix=0.2, nbreset=0, iembeddiag=0, sigma_list=None,
        v2e_list=None, max_iter=10000, sx_list=None, sy_list=None,
        sz_list=None, lx_list=None, ly_list=None, lz_list=None,
        utrans_list=None, ldc=0, u_avg_list=None, j_avg_list=None,
        nelf_list=None, rotations_list=None, lie_odd_params_list=None,
        lie_even_params_list=None, jgenerator_list=None,
        sp_rotations_list=None,
        nval_bot_list=[0], nval_top_list=[2]):

    if '-n' in sys.argv:
        max_iter = int(sys.argv[sys.argv.index('-n') + 1])

    with h5py.File('GPARAM.h5', 'w') as f:
        f['/iso'] = np.asarray([iso])
        f['/ispin'] = np.asarray([ispin])
        f['/num_imp'] = np.asarray([len(ityp_list)])
        f['/ITYP_IMP'] = np.asarray(ityp_list) + 1
        f['/IMAP_IMP'] = np.asarray(imap_list) + 1
        f['/na2_imp'] = np.asarray(na2_list)
        f['/gimix'] = np.asarray([imix])
        f['/gamix'] = [amix]
        f['/gnbreset'] = np.asarray([nbreset])
        f['/giembeddiag'] = np.asarray([iembeddiag])
        f['/gmaxiter'] = np.asarray([max_iter])
        f['/dc_mode'] = np.asarray([ldc])

        f['/dc_u_avg'] = u_avg_list
        f['/dc_j_avg'] = j_avg_list
        if nelf_list is not None:
            f['/dc_nelf_list'] = nelf_list

        if nval_bot_list is not None:
            f['/nval_bot_ityp'] = np.asarray(nval_bot_list)
            f['/nval_top_ityp'] = np.asarray(nval_top_list)

        # for sigma_list, v2e_list
        dim_hs_list = []
        for i, na2 in enumerate(na2_list):
            if sigma_list is None:
                sigma = np.arange(1,na2**2+1).reshape((na2,na2))
            else:
                sigma = sigma_list[i]

            f['/IMPURITY_'+str(i+1)+'/SIGMA_STRUCT'] = sigma.T
            hermitian_mb = sigmatomatrixbasis(sigma)
            dim_hs_list.append(len(hermitian_mb))
            f['/IMPURITY_'+str(i+1)+'/HS'] = np.swapaxes(hermitian_mb,1,2)

            if v2e_list is None:
                v2e = np.zeros((na2,na2,na2,na2), dtype=np.complex)
            else:
                v2e = v2e_list[i]
            f['/IMPURITY_'+str(i+1)+'/V2E'] = v2e.T

            if sx_list is not None:
                f['/IMPURITY_'+str(i+1)+'/SX'] = sx_list[i].T
            if sy_list is not None:
                f['/IMPURITY_'+str(i+1)+'/SY'] = sy_list[i].T
            if sz_list is not None:
                f['/IMPURITY_'+str(i+1)+'/SZ'] = sz_list[i].T
            if lx_list is not None:
                f['/IMPURITY_'+str(i+1)+'/LX'] = lx_list[i].T
            if ly_list is not None:
                f['/IMPURITY_'+str(i+1)+'/LY'] = ly_list[i].T
            if lz_list is not None:
                f['/IMPURITY_'+str(i+1)+'/LZ'] = lz_list[i].T
            if utrans_list is not None:
                f['/IMPURITY_'+str(i+1)+'/DB_TO_SAB']\
                        = utrans_list[i].T
            if rotations_list is not None:
                f['/IMPURITY_'+str(i+1)+'/rotations'] = rotations_list[i]
            if lie_odd_params_list is not None:
                f['/IMPURITY_'+str(i+1)+'/lie_odd_params'] = \
                        lie_odd_params_list[i]
                f['/IMPURITY_'+str(i+1)+'/lie_even_params'] = \
                        lie_even_params_list[i]
                f['/IMPURITY_'+str(i+1)+'/JGENERATOR'] = \
                        np.swapaxes(jgenerator_list[i],1,2)
                f['/IMPURITY_'+str(i+1)+'/SP_ROTATIONS'] = \
                        np.swapaxes(sp_rotations_list[i],1,2)
                f['/IMPURITY_'+str(i+1)+'/nsym_odd'] = \
                        np.asarray(sp_rotations_list[i]).shape[0:1]

        f['/dim_hs_imp'] = np.asarray(dim_hs_list)


def save_gparambands(kpt_wt, nelectron, nbmax, iso=1, ispin=1,
        symop=1, symie=1, ensemble=0,
        ismear=0, delta=0.01, ne_list=None, h1e_list=None):
    with h5py.File('GPARAMBANDS.h5', 'w') as f:
        f['/iso'] = np.asarray([iso])
        f['/ispin'] = np.asarray([ispin])
        f['/kptdim'] = np.asarray(list(kpt_wt.shape))
        f['/nbmax'] = np.asarray([nbmax])
        if ne_list is not None:
            assert ne_list.shape[0] == kpt_wt.shape[0], \
                ' error: ne_list and kpt_wt do not match!'
            f['/NE_LIST'] = np.asarray(ne_list)  # shape (nk,3)
        f['/kptwt'] = kpt_wt
        f['/ismear'] = np.asarray([ismear])
        f['/ensemble'] = [ensemble]
        f['/delta'] = [delta]
        f['/nelectron'] = [nelectron]
        f['/symnop'] = np.asarray([symop])
        f['/symie'] = np.asarray([symie])
        if h1e_list is None:
            h1e_list = [np.zeros((2, 2), dtype=np.complex)]
        for i, h1e in enumerate(h1e_list):
            # Fortran convention.
            f['/IMPURITY_' + str(i + 1) + '/H1E'] = h1e.T



import sys, h5py, subprocess, os, numpy

def write_text_2d(fname, head, dm):
    '''Write text file of 2d arrays.
    '''
    with open(fname, 'w') as f:
        f.write(head)
        for i, dm1 in enumerate(dm):
            for j, dm2 in enumerate(dm1):
                if abs(dm2) > 1.e-12:
                    f.write('{:3d}{:3d}{:24.16f}{:24.16f}\n'.format( \
                            i, j, dm2.real, dm2.imag))


def write_text_4d(fname, head, dm):
    '''Write text file of 4d arrays.
    '''
    with open(fname, 'w') as f:
        f.write(head)
        for i, dm1 in enumerate(dm):
            for j, dm2 in enumerate(dm1):
                for k, dm3 in enumerate(dm2):
                    for l, dm4 in enumerate(dm3):
                        if abs(dm4) > 1.e-12:
                            f.write('{:3d}{:3d}{:3d}{:3d}{:24.16f}{:24.16f}\n' \
                                    .format(i, j, k, l, dm4.real, dm4.imag))


def h5gen_text_embed_hamil(imp):
    '''Generate text files of the embedding Hamiltonian.
    '''
    with h5py.File('EMBED_HAMIL_{}.h5'.format(imp), 'r') as f:
        daalpha = f['/D'][...].T
        lambdac = f['/LAMBDA'][...].T
        h1e     = f['/H1E'][...].T
        v2e     = f['/V2E'][...].T

    # h1e file
    head = '# list of h1e_{alpha, beta} of significance \n' + \
            '# alpha   beta            h1e.real            h1e.imag \n'
    write_text_2d('h1e_{}.dat'.format(imp), head, h1e)

    # d file
    head = '# list of d_{a, alpha} with significance \n' + \
            '#     a  alpha              d.real              d.imag \n'
    write_text_2d('d_aalpha_{}.dat'.format(imp), head, daalpha)

    # lambdac file
    head = '# list of lambdac_{a, b} of significance \n' + \
            '#     a      b        lambdac.real        lambdac.imag \n'
    write_text_2d('lambdac_{}.dat'.format(imp), head, lambdac)

    # v2e file
    head = '# Note H_loc = \sum_{a,b} {h1e_a,b c_a^\dagger c_b} \n' + \
            '#            + 1/2 \sum_{a, b, c, d} v2e_{a,b,c,d}  \n' + \
            '#            * c_a^\dagger c_c^\dagger c_d c_b \n' + \
            '# list of v2e_{alpha, beta, gamma, delta} of significance \n' + \
            '# alpha   beta  gamma  delta              v2e.real' + \
            '              v2e.imag\n'
    if not os.path.isfile('v2e_{}.dat'.format(imp)):
        write_text_4d('v2e_{}.dat'.format(imp), head, v2e)


def gen_file_lat(imp, flat, reorder=None):
    '''Generate lat file.
    '''
    command = ['/usr/bin/time', '-v', '-a', '-o', 'timing_{}'.format(imp),
            'syten-mps-embedding-lanata', '--h1e', './h1e_{}.dat'.format(imp),
            '--d', './d_aalpha_{}.dat'.format(imp), '--lambdac',
            './lambdac_{}.dat'.format(imp), '--v2e', './v2e_{}.dat'.format(imp),
            '-o', flat, '-q']
    if reorder is not None:
        command.extend(['-r', reorder])
    print(' '.join(command))
    subprocess.call(command)


def gen_file_rnd_state(imp, norbs, lat):
    '''Generate random state for initialization.
    '''
    command = ['/usr/bin/time', '-v', '-a', '-o', 'timing_{}'.format(imp),
            'syten-random', '-s', str(norbs), '-l', lat, '-o',
            'rnd_{}.state'.format(imp)]
    print(' '.join(command))
    subprocess.call(command)


def run_syten_dmrg(imp, lat, inp_state, out_f, s_config, out_state=None,
        threads_tensor=1):
    '''Run dmrg calculation.
    '''
    command = ['/usr/bin/time', '-v', '-a', '-o', 'timing_{}'.format(imp),
            'syten-dmrg', '-l', lat, '-i', inp_state, '-o', out_f,
            '-s', s_config, '-q']
    if out_state is not None:
        command.extend(['-f', out_state])
    if threads_tensor > 1:
        command.extend(['--threads-tensor', str(threads_tensor)])
    print(' '.join(command))
    subprocess.call(command)


def run_syten_expectation(imp, lat, state, f_expval):
    '''Calculate expectation values.
    '''
    command = ['syten-expectation', '-c', '-a', state, '-l', lat, \
            '--template-file', f_expval, '-q']
    return subprocess.check_output(command)


def run_syten_mutual_information(imp, state, f_reorder):
    command = ['/usr/bin/time', '-v', '-a', '-o', 'timing_{}'.format(imp),
            'syten-mutualInformation', '-o', state, '-f', f_reorder]
    print(' '.join(command))
    subprocess.call(command)


def driver_dmrg(s_config= \
        '(t 1e-8 d 1e-8 m 50 expandBlocksize 10 x 5 save false)' + \
        '(m 100) '*8 + '(m 100 d 1e-8 save true)'):
    '''driver for the impurity dmrg calculations.
    '''
    if '-i' in sys.argv:
        imp = sys.argv[sys.argv.index('-i')+1]
    else:
        imp = 1

    if os.path.isfile('s_config_{}.cfg'.format(imp)):
        s_config = ''
        for line in open('s_config_{}.cfg'.format(imp), 'r').readlines():
            s_config += line.replace('\n', '')

    with h5py.File('EMBED_HAMIL_{}.h5'.format(imp), 'r') as f:
        na2 = f['/na2'][0]
        lambdac = f['/LAMBDA'][...].T

    # dump to text file due to dmrg code
    h5gen_text_embed_hamil(imp)

    f_reorder = 'reordering_{}'.format(imp)
    if not os.path.isfile(f_reorder):
        # generate lat file
        flat = 'without-reordering_{}.lat'.format(imp)
        gen_file_lat(imp, flat)

        # generate random state for initialization.
        gen_file_rnd_state(imp, na2, flat)

        # initial stage 1/2 sweeping
        flat = 'without-reordering_{}.lat:H1e Hd Hf Hv2e + + +'.format(imp)
        inp_state = 'rnd_{}.state'.format(imp)
        out_f = 'without-reordering-dmrg_{}'.format(imp)
        s_config0 = '(t 1e-8 d 1e-6 m 50 x 5) (m 100)'
        run_syten_dmrg(imp, flat, inp_state, out_f, s_config0)

        # reorder
        state = out_f + '_2_5.state'
        run_syten_mutual_information(imp, state, f_reorder)
    else:
        if os.path.isfile('with-reordering-gs_{}.state'.format(imp)):
            inp_state = 'with-reordering-gs_{}.state'.format(imp)
        else:
            inp_state = 'rnd_{}.state'.format(imp)

    # generate lat file with reordering
    flat = 'with-reordering_{}.lat'.format(imp)
    gen_file_lat(imp, flat, reorder=f_reorder)

    # dmrg after reordering
    flat = 'with-reordering_{}.lat:H1e Hd Hf Hv2e + + +'.format(imp)
    out_f = 'with-reordering-dmrg_{}'.format(imp)
    out_state = 'with-reordering-gs_{}.state'.format(imp)
    run_syten_dmrg(imp, flat, inp_state, out_f, s_config,
            out_state=out_state)

    # get expectation value
    f_temp = 'exp_val_{}.template'.format(imp)
    na4 = na2*2
    if not os.path.isfile(f_temp):
        with open(f_temp, 'w') as f:
            for i in range(na4):
                for j in range(i, na4):
                    f.write('{{ CH_{} C_{} * }}\n'.format(i, j))
            f.write('{ H1e Hd Hf Hv2e + + + }')
    flat = 'with-reordering_{}.lat'.format(imp)
    res = run_syten_expectation(imp, flat, out_state, f_temp)
    res = res.split('\n')[:na4*(na4+1)/2+1]
    dm = numpy.zeros([na4, na4], dtype=numpy.complex)
    ij = 0
    for i in range(na4):
        for j in range(i, na4):
            res_ = map(float, res[ij].split())
            ij += 1
            dm[i, j] = res_[0] + res_[1]*1.j
            if i != j:
                dm[j, i] = numpy.conj(dm[i, j])
    res_ = map(float, res[-1].split())
    assert numpy.abs(res_[1]) < 1.e-6, ' syten: non-real impurity etot!'

    # convention: f_b f_a^\dagger = \delta_{a,b} - f_a^\dagger f_b
    etot = res_[0] - numpy.trace(lambdac)
    with h5py.File('EMBED_HAMIL_RES_{}.h5'.format(imp), 'w') as f:
        f['/emol'] = [etot.real]
        f['/DM'] = dm.T



if __name__ == '__main__':
    driver_dmrg()



from math import pi, sqrt
import numpy as np
try:
    from builtins import zip
except:
    pass

from itertools import product,count
from pygrisb.estructure import fermi
from pygrisb.basic import units


class DOS:

    def __init__(self, w_k, e_skn,  width=0.1, window=None, npts=201):
        """Electronic Density Of States object.

        width: float
          Width of guassian smearing.
        window: tuple of two float
          Use ``window=(emin, emax)``.  If not specified, a window
          big enough to hold all the eigenvalues will be used.
        npts: int
          Number of points.

        """

        self.npts = npts
        self.width = width
        self.w_k = w_k
        self.e_skn = e_skn

        if window is None:
            emin = self.e_skn.min() - 5 * self.width
            emax = self.e_skn.max() + 5 * self.width
        else:
            emin, emax = window

        self.energies = np.linspace(emin, emax, npts)


    def delta(self, energy):
        """
        Return a delta-function centered at energy.
        """
        x = -((self.energies - energy) / self.width)**2
        return np.exp(x) / (sqrt(pi) * self.width)


    def get_energies(self):
        """"
        return energy mesh.
        """
        return self.energies


    def get_dos_component(self, psi_skn_w):
        """
        Get array of DOS values.
        """
        dos_list = []

        # spin
        for e_kn, psi_kn_w in zip(self.e_skn,psi_skn_w):
            dos = np.zeros(self.npts, dtype=np.complex)
            # k-point
            for w, e_n, psi_n_w in zip(self.w_k, e_kn, psi_kn_w):
                # band index
                for e, psi_w in zip(e_n, psi_n_w):
                    dos += w * self.delta(e) * psi_w
            dos_list.append(dos)
        return np.array(dos_list)


    def get_dos_t(self):
        """
        Get array of DOS values.
        """
        dos_list = []

        # spin
        for e_kn in self.e_skn:
            dos = np.zeros(self.npts)
            # k-point
            for w, e_n in zip(self.w_k, e_kn):
                # band index
                for e in e_n:
                    dos += w * self.delta(e)
            dos_list.append(dos)
        return np.array(dos_list)


    def get_dos_noncoherent(self, psi_sksn, ecdfdffdelta, f_skn, bnd_ne,
            i):
        """
        Non-coherent part of dos.
        """

        # symmetry operations
        s_range = range(psi_sksn.shape[2])
        norb = ecdfdffdelta.shape[0]

        dos_list = []
        # spin
        for e_kn, f_kn, psi_ksn in zip(self.e_skn, f_skn, psi_sksn):
            dos = np.zeros(self.npts, dtype=np.complex)

            # k-point
            for e_n1, f_n1, psi_sn1, nb1 in zip(e_kn, f_kn, psi_ksn, bnd_ne):
                f_k1 = f_n1[0]
                for n1, e1, f1 in zip(count(),
                        e_n1[nb1[1]:nb1[2]+1],
                        f_n1[nb1[1]:nb1[2]+1]):
                    if(abs(e1) > 10): continue
                    for n2, e2, f2 in zip(count(),
                            e_n1[nb1[1]:nb1[2]+1],
                            f_n1[nb1[1]:nb1[2]+1]):
                        if(abs(e2) > 10): continue
                        for n3, e3, f3 in zip(count(),
                                e_n1[nb1[1]:nb1[2]+1],
                                f_n1[nb1[1]:nb1[2]+1]):
                            if(abs(e3) > 10): continue
                            res1 = (f_k1-f1)*(f_k1-f2)*f3 + f1*f2*(f_k1-f3)
                            if(abs(res1)<1.e-16):
                                continue
                            if(abs(f1/f_k1 -0.5) > 0.0001 and
                                    abs(f2/f_k1 -0.5) > 0.0001 and
                                    abs(f3/f_k1 -0.5) > 0.0001):
                                continue

                            res1 *= self.delta(e1+e2-e3)
                            for s1 in s_range:
                                resi = np.einsum('abc,b,c,a',
                                        ecdfdffdelta[i],
                                        psi_sn1[s1,n1,:norb]\
                                        .conj(),
                                        psi_sn1[s1,n2,:norb] \
                                        .conj(),
                                        psi_sn1[s1,n3,:norb])
                                dos += res1*np.conj(resi)*resi
            dos_list.append(dos)
        return np.array(dos_list)


    def get_dos_noncoherent_all(self, psi_sksn, ecdfdffdelta, f_skn, bnd_ne,
            i, j):
        """
        Non-coherent part of dos.
        """

        # symmetry operations
        s_range = range(psi_sksn.shape[2])
        norb = ecdfdffdelta.shape[0]

        dos_list = []
        # spin
        for e_kn, f_kn, psi_ksn in zip(self.e_skn, f_skn, psi_sksn):
            dos = np.zeros(self.npts, dtype=np.complex)

            # k-point
            for e_n1,f_n1, psi_sn1, nb1 in zip(e_kn, f_kn, psi_ksn, bnd_ne):
                # k-point
                for e_n2, f_n2, psi_sn2, nb2 in zip(e_kn, f_kn, psi_ksn,
                        bnd_ne):
                    # k-point
                    for e_n3, f_n3, psi_sn3, nb3 in zip(e_kn, f_kn, psi_ksn,
                            bnd_ne):
                        for n1, e1, f1 in zip(count(),
                                e_n1[nb1[1]:nb1[2]+1],
                                f_n1[nb1[1]:nb1[2]+1]):
                            for n2, e2, f2 in zip(count(),
                                    e_n2[nb2[1]:nb2[2]+1],
                                    f_n2[nb2[1]:nb2[2]+1]):
                                    for n3, e3, f3 in zip(count(),
                                            e_n3[nb3[1]:nb3[2]+1],
                                            f_n3[nb3[1]:nb3[2]+1]):
                                        res1 = (1-f1)*(1-f2)*f3 + f1*f2*(1-f3)
                                        res1 *= self.delta(e1+e2-e3)
                                        for s1, s2, s3 in product(s_range,
                                                s_range,s_range):
                                            resi = np.einsum('abc,b,c,a',
                                                    ecdfdffdelta[i],
                                                    psi_sn1[s1,n1,:norb]\
                                                    .conj(),
                                                    psi_sn2[s2,n2,:norb] \
                                                    .conj(),
                                                    psi_sn3[s3,n2,:norb])
                                            resj = np.einsum('abc,b,c,a',
                                                    ecdfdffdelta[j],
                                                    psi_sn1[s1,n1,:norb] \
                                                    .conj(),
                                                    psi_sn2[s2,n2,:norb] \
                                                    .conj(),
                                                    psi_sn3[s3,n3,:norb])
                                            dos += res1*np.conj(resi)*resj
            dos_list.append(dos)
        return np.array(dos_list)


def get_all_psi_skn_w(e_skn, psi_sksn, bnd_ne):
    '''
    Reduce psi_sksn to psi_skn_w.
    '''
    psi_skn_w = np.zeros(list(e_skn.shape) + [psi_sksn.shape[-1]], dtype=float)
    for isp in range(e_skn.shape[0]):
        for k in range(e_skn.shape[1]):
            for isy in range(psi_sksn.shape[2]):
                for n in range(bnd_ne[k, 1], bnd_ne[k, 2]):
                    for a in range(psi_sksn.shape[-1]):
                        psi_skn_w[isp, k, n, a] += (
                                psi_sksn[isp, k, isy, n - bnd_ne[k, 1], a] *
                                np.conj(
                                psi_sksn[isp, k, isy, n - bnd_ne[k, 1], a])).\
                                real
    return psi_skn_w / psi_sksn.shape[2]


def get_all_psi_skn_w_ab(e_skn, psi_sksn, bnd_ne):
    '''
    Reduce psi_sksn to psi_skn_w_ab.
    '''
    n_orb = psi_sksn.shape[-1]
    psi_skn_w_ab = np.zeros(list(e_skn.shape) + [n_orb, n_orb], dtype=complex)
    for k in range(e_skn.shape[1]):
        for n in range(bnd_ne[k, 1], bnd_ne[k, 2]):
            psi_skn_w_ab[:, k, n, :, :] = np.einsum('ijk, ijl->ikl',
                    psi_sksn[:, k, :, n - bnd_ne[k, 1], :],
                    np.conj(
                    psi_sksn[:, k, :, n - bnd_ne[k, 1], :]))
    return psi_skn_w_ab / psi_sksn.shape[2]



if __name__ == "__main__":
    '''
    Test run.
    '''
    import h5py

    with h5py.File("GPARAMBANDS.h5", 'r') as f:
        w_k = f["/kptwt"][...]
        bnd_ne = f["/NE_LIST"][...]
        nsym = f["/symnop"][...]
        nkp = len(w_k)

    with h5py.File("GLOG.h5", 'r') as f:
        e_fermi = f["/e_fermi"][0]
        nspin = f["/nspin"][0]

    with h5py.File("GBANDS_0.h5", 'r') as f:
        e_skn = []
        for isp in range(nspin):
            e_kn = []
            for k in range(nkp):
                e_kn.append(f['/ISPIN_{}/IKP_{}/ek'.format(isp+1,k+1)][...])
            e_skn.append(e_kn)

    num_elec = np.sum(f_skn) - 2
    print(fermi.num_electron_diff(0.0, 0.001, e_skn, w_k, bnd_ne[:,0], num_elec))

    # test fermi level
    from scipy.optimize import newton
    delta_list = [10.0, 8.0, 6.0, 4.0, 3.0, 2.0, 1.0, 0.9, 0.8, 0.6, 0.5, 0.3, 0.2, 0.1, 0.01]
    f_list = []
    x0 = 0
    for delta in delta_list:
        x0 = newton(fermi.num_electron_diff, x0, \
                args = (delta, e_skn, w_k, bnd_ne[:,0], num_elec))
        print(x0)

    psi_skn_w_ab = get_all_psi_skn_w_ab(e_skn, psi_sksn, bnd_ne)
    window = (-5., 15.)
    dos = DOS(w_k, e_skn,  width=0.05, window=window, npts=1001)
    energies = dos.get_energies()
    dos_t = dos.get_dos_t()

    # dos_f
    psi_sksn_f = np.einsum('...ii', psi_skn_w_ab[...,:,:])
    dos_f = dos.get_dos_component(psi_sksn_f)

    import matplotlib.pyplot as plt

    plt.figure(0)
    plt.plot(energies, dos_t[0].real, '-', label='t')
    plt.plot(energies, dos_f[0].real, 'o', label='f')
    plt.legend()
    plt.xlabel("E (eV)")
    plt.ylabel("DOS (states/f.u.)")
#    plt.savefig('dos.pdf')
    plt.show()



from __future__ import print_function

import numpy as np
import h5py
from scipy.linalg import block_diag
from pygrisb.io.fio import file_exists


def get_orbital_transformation(l, nspin):
    '''give the unitary transformation from wien2k basis to the standard
    complex spherical Harmonics with Condon-shortley phase.
    '''

    u_trans = np.identity((2*l+1), dtype=complex)

    # Test for the weird phase (identified in NiO AFM case.)
    if l == 2:
        u_trans[1,1] = -1.j
        u_trans[3,3] =  1.j

    if nspin == 2:
        u_trans = block_diag(u_trans, u_trans)
    return u_trans


def h4set_indmfl(emin=-10., emax=10., projtype=-2):
    '''create case.indmfl for the interface given file ginit.h5.
    '''
    finit = h5py.File('ginit.h5', 'r')
    if '/struct/case' not in finit:
        raise ValueError('path /struct/case does not exist in ginit.h5!')

    case = finit['/struct/case'][()].split('.')[0]

    if file_exists(case+'.indmfl'):
        print(case+'.indmfl exists! \n' +
                'Please rm it if you intend to generate a new one.')
        return

    findm = open(case+'.indmfl', 'w')
    findm.write(('{:6.2f} {:6.2f} {:3d}  # hybrid. emin/max w.r.t. FS,' + \
            ' projector type\n').format(emin, emax, projtype))

    atom_symbols = finit['/struct/symbols'][()]
    atom_symbols = [s.decode() for s  in atom_symbols]
    unique_corr_atom_symbols = finit['/usrqa/unique_corr_symbol_list'][()]
    unique_corr_atom_symbols = [s.decode() for s in unique_corr_atom_symbols]

    n_corr_atoms = 0
    for symbol in atom_symbols:
        if symbol in unique_corr_atom_symbols:
            n_corr_atoms += 1

    findm.write('{:2d}                 # number of correlated atoms\n'. \
            format(n_corr_atoms))

    if 'y' in finit['/usrqa/spin_orbit_coup'][()]:
        nspin = 2
    else:
        nspin = 1
    unique_df_list = finit['/usrqa/unique_df_list'][()]
    unique_df_list = [df.decode() for df in unique_df_list]

    from pygrisb.symm.angular_momentum_1p import get_l_list_from_string
    cix = 0
    norbsmax = 0
    l_super_list = []
    for i, symbol in enumerate(atom_symbols):
        if symbol not in unique_corr_atom_symbols:
            continue
        df = unique_df_list[unique_corr_atom_symbols.index(symbol)]
        l_list = get_l_list_from_string(df)

        findm.write('{:2d} {:2d}              # iatom, num_L\n'. \
                format(i+1, len(l_list)))
        l_super_list.extend(l_list)
        for l in l_list:
            cix += 1
            findm.write(' {:2d} {:2d} {:2d}          # L, qsplit, cix\n'.\
                    format(l, 3, cix))
            norbsmax = max(norbsmax, (2*l+1)*nspin)

    findm.write('----- unitary transformation for correlated orbitals -----\n')
    findm.write(('{:2d} {:2d}              # num indep. kcix blocks,' + \
            ' max dimension\n').format(cix, norbsmax))
    for i, l in enumerate(l_super_list):
        norbs = (2*l+1)*nspin
        findm.write('{:2d} {:2d}              # cix, dimension\n'. \
                format(i+1, norbs))
        findm.write('----- unitary transformation matrix -----\n')
        u_trans = get_orbital_transformation(l, nspin)
        for u1 in u_trans:
            for u12 in u1:
                findm.write('{:20.16f}{:20.16f} '.format(u12.real, u12.imag))
            findm.write('\n')


if __name__=='__main__':
    h4set_indmfl()



# tranform the GPARAM.h5 file to local basis of
# relativistic spherical harmonics.
import h5py,numpy,os
from pygrisb.math.matrix_util import unitary_transform_coulomb_matrix
from pygrisb.symm.angular_momentum_1p import get_JU_relat_sph_harm_cg



def trans_gparam(u_list):
    with h5py.File("GPARAM.h5", "a") as f:
        for i,u in enumerate(u_list):
            u_db2sab = f["/IMPURITY_{}/DB_TO_SAB".format(i+1)][()].T
            u_db2sab = u_db2sab.dot(u)
            f["/IMPURITY_{}/DB_TO_SAB".format(i+1)][...]=u_db2sab.T
            hs_list = f["/IMPURITY_{}/HS".format(i+1)][()].swapaxes(1,2)
            hs_list = [u.T.conj().dot(hs).dot(u) for hs in hs_list]
            f["/IMPURITY_{}/HS".format(i+1)][...]=numpy.asarray( \
                    hs_list).swapaxes(1,2)
            for entry in ["LX", "LY", "LZ", "SX", "SY", "SZ"]:
                op = f["/IMPURITY_{}/{}".format(i+1,entry)][()].T
                op = u.T.conj().dot(op).dot(u)
                f["/IMPURITY_{}/{}".format(i+1,entry)][...] = op.T
            # to be safe, set self-energy form to be random.
            f["/IMPURITY_{}/SIGMA_STRUCT".format(i+1)][...] = \
                    (numpy.arange(len(u)**2)+1).reshape(len(u),len(u))
            v2e = f["/IMPURITY_{}/V2E".format(i+1)][()].T
            v2e = unitary_transform_coulomb_matrix(v2e, u)
            f["/IMPURITY_{}/V2E".format(i+1)][...]=v2e.T
    if os.path.isfile("WH_RL_INP.h5"):
        with h5py.File("WH_RL_INP.h5", "a") as f:
            for i,u in enumerate(u_list):
                for entry in ["LAMBDA", "R"]:
                    a = f["/IMPURITY_{}/{}".format(i+1, entry)][()].T
                    a = u.T.conj().dot(a).dot(u)
                    f["/IMPURITY_{}/{}".format(i+1, entry)][...] = a.T



def trans_gparam_rsh():
    '''transform GPARAM.h5 to relativisitic spherical harmonics basis.
    '''
    l_from_dimms = {2:0, 6:1, 10:2, 14:3}
    with h5py.File("GPARAM.h5", "a") as f:
        nimp = f["/num_imp"][0]
        u_list = []
        for i in range(nimp):
            u = f["/IMPURITY_{}/DB_TO_SAB".format(i+1)][()].conj()
            l_list = [l_from_dimms[len(u)]]
            _, u_csh2rsh = get_JU_relat_sph_harm_cg(l_list)
            u = u.dot(u_csh2rsh)
            u_list.append(u)
    trans_gparam(u_list)



if __name__ == "__main__":
    trans_gparam_rsh()
